/**
 * APM X-Gene DDR3 controller code
 *
 * Copyright (c) 2013, AppliedMicro Corp. All rights reserved.
 * All rights reserved. Feng Kan <fkan@apm.com>.
 *
 * This program and the accompanying materials
 * are licensed and made available under the terms and conditions of the BSD License
 * which accompanies this distribution.  The full text of the license may be found at
 * http://opensource.org/licenses/bsd-license.php
 *
 * THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
 * WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
 *
 **/

#ifndef _DDR_MCU_H
#define _DDR_MCU_H

#define MCU_SUPPORTED_RANKS 8U
#define PHY_SUPPORTED_LANES 9U

#define MCU_T_CRL_MIN 5
#define MCU_T_CRL_MAX 18
#define MCU_T_CWL_MIN 4
#define MCU_T_CWL_MAX 13

#define MCU_MRS 0x0
#define MCU_REF 0x1
#define MCU_PRE 0x2
#define MCU_ACT 0x3
#define MCU_WR 0x4
#define MCU_RD 0x5
#define MCU_ZQC 0x6
#define MCU_NOP 0x7
#define MCU_REQUEST_ID_RANGE 0x4
#define MCU_REQUEST_COLUMN_RANGE 0xa
#define MCU_REQUEST_ROW_RANGE 0x18
#define MCU_REQUEST_BANK_RANGE 0x28
#define MCU_REQUEST_RANK_RANGE 0x2b
#define MCU_REQUEST_RANK_BANK_RANGE 0x28
#define MCU_REQUEST_COLSN_VALID_RANGE 0x2e
#define MCU_REQUEST_COLSN_LINK_RANGE 0x2f
#define MCU_REQUEST_DATA_PTR_RANGE 0x32
#define MCU_BANK_DATA_PTR_RANGE 0x28
#define MCU_BANK_ROW_RANGE 0x16
#define MCU_BANK_COLUMN_RANGE 0x8
#define MCU_BANK_ID_RANGE 0x2
#define MCU_RESET_FSM_ST_RNG 0x0
#define MCU_RESET_FSM_stINITSTART 0x0
#define MCU_RESET_FSM_stWAIT 0x1
#define MCU_RESET_FSM_stWAITNOP 0x2
#define MCU_RESET_FSM_stRCENCHK 0x3
#define MCU_RESET_FSM_stRCWRITE 0x4
#define MCU_RESET_FSM_stRCCHK 0x5
#define MCU_RESET_FSM_stTXPR 0x6
#define MCU_RESET_FSM_stMRCHK 0x7
#define MCU_RESET_FSM_stMR2 0x8
#define MCU_RESET_FSM_stMR3 0x9
#define MCU_RESET_FSM_stMR1 0xa
#define MCU_RESET_FSM_stMR0 0xb
#define MCU_RESET_FSM_stZQCHK 0xc
#define MCU_RESET_FSM_stZQC 0xd
#define MCU_RESET_FSM_stWRCHK 0xe
#define MCU_RESET_FSM_stWRLVLCHK 0xf
#define MCU_RESET_FSM_stWRLVLODT 0x10
#define MCU_RESET_FSM_stMRWL 0x11
#define MCU_RESET_FSM_stWRLVLEN 0x12
#define MCU_RESET_FSM_stWLSTROBE 0x13
#define MCU_RESET_FSM_stWLSTROBEPHY 0x14
#define MCU_RESET_FSM_stWLRESPWAITP 0x15
#define MCU_RESET_FSM_stWLLOAD 0x16
#define MCU_RESET_FSM_stWLRESP 0x17
#define MCU_RESET_FSM_stWLSTEP 0x18
#define MCU_RESET_FSM_stMR1R 0x19
#define MCU_RESET_FSM_stMR3ON 0x1a
#define MCU_RESET_FSM_stGTLVLPHYRD 0x1b
#define MCU_RESET_FSM_stGTRESPWAITP 0x1c
#define MCU_RESET_FSM_stGTSTEPP 0x1d
#define MCU_RESET_FSM_stGTLVLEN 0x1e
#define MCU_RESET_FSM_stGTLOAD 0x1f
#define MCU_RESET_FSM_stGTSTROBE 0x20
#define MCU_RESET_FSM_stGTRESP 0x21
#define MCU_RESET_FSM_stGTSTEP 0x22
#define MCU_RESET_FSM_stRDCHK 0x23
#define MCU_RESET_FSM_stRDLVLPHYRD1 0x24
#define MCU_RESET_FSM_stRDLVLPHYRD2 0x25
#define MCU_RESET_FSM_stRDRESPWAITP 0x26
#define MCU_RESET_FSM_stRDSTEPP 0x27
#define MCU_RESET_FSM_stRDLVLEN 0x28
#define MCU_RESET_FSM_stRDLOAD 0x29
#define MCU_RESET_FSM_stRDSTROBE 0x2a
#define MCU_RESET_FSM_stRDRESP 0x2b
#define MCU_RESET_FSM_stRDSTEP 0x2c
#define MCU_RESET_FSM_stMR3OFF 0x2d
#define MCU_RESET_FSM_stPHYPROG 0x2e
#define MCU_RESET_FSM_stBISTCHK 0x2f
#define MCU_RESET_FSM_stBISTACT 0x30
#define MCU_RESET_FSM_stBISTWRCAS0 0x31
#define MCU_RESET_FSM_stBISTWRCAS1 0x32
#define MCU_RESET_FSM_stBISTRDCAS0 0x33
#define MCU_RESET_FSM_stBISTRDCAS1 0x34
#define MCU_RESET_FSM_stBISTGAP 0x35
#define MCU_RESET_FSM_stBISTCOMPARE 0x36
#define MCU_RESET_FSM_stBISTPRE 0x37
#define MCU_RESET_FSM_stCUCHK 0x38
#define MCU_RESET_FSM_stCTLUPDATE 0x39
#define MCU_RESET_FSM_stRESTART 0x3a
#define MCU_RESET_FSM_stISSUEOP0 0x3b
#define MCU_RESET_FSM_stISSUEOP1 0x3c
#define MCU_RESET_FSM_stISSUEOP2 0x3d
#define MCU_RESET_FSM_stMEMINITDONE 0x3e
#define MCU_RESET_FSM_stSPARE 0x3f
#define MCU_RSTSTS_BISTERR_RNG 0x1f
#define MCU_RSTSTS_ICST_RNG 0x18
#define MCU_RSTSTS_RSVD1_RNG 0x17
#define MCU_RSTSTS_RETST_RNG 0x10
#define MCU_RSTSTS_BADSTATE_RNG 0xf
#define MCU_RSTSTS_TOERR_RNG 0xe
#define MCU_RSTSTS_RSVD2_RNG 0xd
#define MCU_RSTSTS_PREREFRESHOVER_RNG 0xc
#define MCU_RSTSTS_CLK200OVER_RNG 0xb
#define MCU_RSTSTS_CLK500OVER_RNG 0xa
#define MCU_RSTSTS_TXPROVER_RNG 0x9
#define MCU_RSTSTS_RCOVER_RNG 0x8
#define MCU_RSTSTS_MROVER_RNG 0x7
#define MCU_RSTSTS_ZQCLOVER_RNG 0x6
#define MCU_RSTSTS_WRLVLOVER_RNG 0x5
#define MCU_RSTSTS_GATEOVER_RNG 0x4
#define MCU_RSTSTS_RDLVLOVER_RNG 0x3
#define MCU_RSTSTS_BISTOVER_RNG 0x2
#define MCU_RSTSTS_CTLUPDTOVER_RNG 0x1
#define MCU_RSTSTS_DDR_MEM_INIT_DONE_RNG 0x0

/*
 * MCU register definition
 */
/* Init Ctl bits */
#if 0
#define MCU_DDR_INIT_CTL_CALIBGO_RNG            31
#define MCU_DDR_INIT_CTL_RESERVED_RNG           30:12
#define MCU_DDR_INIT_CTL_RCWAIT_RNG             11:8
#define MCU_DDR_INIT_CTL_RESERVED1_RNG          7
#define MCU_DDR_INIT_CTL_REFRESHCNT_RNG         6:4
#define MCU_DDR_INIT_CTL_RESERVED2_RNG          3:1
#define MCU_DDR_INIT_CTL_GO2RESET_RNG           0

/* Init Seq bits for MCU_DDR_INIT_SEQ */
#define MCU_DDR_INIT_SEQ_RESERVED_RNG           31:13
#define MCU_DDR_INIT_SEQ_PREREFRESHENAB_RNG     12
#define MCU_DDR_INIT_SEQ_CLK200ENAB_RNG         11
#define MCU_DDR_INIT_SEQ_CLK500ENAB_RNG         10
#define MCU_DDR_INIT_SEQ_TXPRENAB_RNG           9
#define MCU_DDR_INIT_SEQ_RCENAB_RNG             8
#define MCU_DDR_INIT_SEQ_MRENAB_RNG             7
#define MCU_DDR_INIT_SEQ_ZQCLENAB_RNG           6
#define MCU_DDR_INIT_SEQ_WRLVLENAB_RNG          5
#define MCU_DDR_INIT_SEQ_GATEENAB_RNG           4
#define MCU_DDR_INIT_SEQ_RDLVLENAB_RNG          3
#define MCU_DDR_INIT_SEQ_BISTENAB_RNG           2
#define MCU_DDR_INIT_SEQ_CTLUPDATE_RNG          1
#define MCU_DDR_INIT_SEQ_SETMEMINITDONE_RNG     0
/* Mcu FSM Status bits */
#define MCU_DDR_INIT_STATUS_BISTERROR_RNG               31
#define MCU_DDR_INIT_STATUS_INITCTLSTATE_RNG            30:24
#define MCU_DDR_INIT_STATUS_RESERVED_RNG                23
#define MCU_DDR_INIT_STATUS_RETURNSTATE_RNG             22:16
#define MCU_DDR_INIT_STATUS_BADSTATE_RNG                15
#define MCU_DDR_INIT_STATUS_TIMEOUTERR_RNG              14
#define MCU_DDR_INIT_STATUS_RSVD2_RNG           13
#define MCU_DDR_INIT_STATUS_PREREFRESHDONE_RNG          12
#define MCU_DDR_INIT_STATUS_CLK200DONE_RNG              11
#define MCU_DDR_INIT_STATUS_CLK500DONE_RNG              10
#define MCU_DDR_INIT_STATUS_TXPRDONE_RNG                9
#define MCU_DDR_INIT_STATUS_RCDONE_RNG          8
#define MCU_DDR_INIT_STATUS_MRDONE_RNG          7
#define MCU_DDR_INIT_STATUS_ZQCLDONE_RNG                6
#define MCU_DDR_INIT_STATUS_WRLVLDONE_RNG               5
#define MCU_DDR_INIT_STATUS_GATEDONE_RNG                4
#define MCU_DDR_INIT_STATUS_RDLVLDONE_RNG               3
#define MCU_DDR_INIT_STATUS_BISTDONE_RNG                2
#define MCU_DDR_INIT_STATUS_CTLUPDATEDONE_RNG           1
#define MCU_DDR_INIT_STATUS_MEMINITDONE_RNG             0 
#endif

/* THIS IS ATUOMATICALLY GENERATE CODE. PLEASE DO NOT MODIFY IT MANUALLY */
#define MCU_REG_HPWEIGHT (0x0U<<2)
#define MCU_REG_LPWEIGHT (0x1U<<2)
#define MCU_REG_WPWEIGHT (0x2U<<2)
#define MCU_REG_MAXINTERVAL (0x3U<<2)
#define MCU_REG_QUANTUMSIZE (0x4U<<2)
#define MCU_REG_AGETHRESH_HP (0x5U<<2)
#define MCU_REG_AGETHRESH_LP (0x6U<<2)
#define MCU_REG_AGETHRESH_WP (0x7U<<2)
#define MCU_REG_PAGE_MODE (0x8U<<2)
#define MCU_REG_HASH_CTL (0x9U<<2)
#define MCU_REG_DIMM_ADDR_MIRROR_CTL (0xcU<<2)
#define MCU_REG_FAST_CPU_ENABLE (0xdU<<2)
#define MCU_REG_BANK_ACTPRE (0xeU<<2)
#define MCU_REG_BANK_RDPRE (0xfU<<2)
#define MCU_REG_BANK_WRPRE (0x10U<<2)
#define MCU_REG_BANK_PREACT (0x11U<<2)
#define MCU_REG_BANK_ACTACT (0x12U<<2)
#define MCU_REG_BANK_RDAACT (0x13U<<2)
#define MCU_REG_BANK_WRAACT (0x14U<<2)
#define MCU_REG_BANK_ACTRD (0x15U<<2)
#define MCU_REG_BANK_ACTWR (0x16U<<2)
#define MCU_REG_BANK_ACTRDA (0x17U<<2)
#define MCU_REG_BANK_ACTWRA (0x18U<<2)
#define MCU_REG_RANK_ACTACT (0x19U<<2)
#define MCU_REG_RANK_RDRD (0x1aU<<2)
#define MCU_REG_RANK_WRWR (0x1bU<<2)
#define MCU_REG_RANK_RDWR (0x1cU<<2)
#define MCU_REG_RANK_WRRD (0x1dU<<2)
#define MCU_REG_RANK_ACTFAW (0x1eU<<2)
#define MCU_REG_SYS_RDRD (0x1fU<<2)
#define MCU_REG_SYS_WRWR (0x20U<<2)
#define MCU_REG_SYS_RDWR (0x21U<<2)
#define MCU_REG_SYS_WRRD (0x22U<<2)
#define MCU_REG_RANK_REFACTREF (0x23U<<2)
#define MCU_REG_REFINT (0x24U<<2)
#define MCU_REG_REFRESH_BURST_CNT (0x26U<<2)
#define MCU_REG_T_WR_LATENCY (0x27U<<2)
#define MCU_REG_T_RD_LATENCY (0x28U<<2)
#define MCU_REG_BY4_MODE (0x29U<<2)
#define MCU_REG_DDR_CLKDISABLE (0x2aU<<2)
#define MCU_REG_UPDATECTL (0x2bU<<2)
#define MCU_REG_PQDEBUGCTL (0x2cU<<2)
#define MCU_REG_PQDEBUGDATAHI (0x2dU<<2)
#define MCU_REG_PQDEBUGDATALO (0x2eU<<2)
#define MCU_REG_MCUIDLESTS (0x2fU<<2)
#define MCU_REG_MRS_0_VALUE (0x34U<<2)
#define MCU_REG_MRS_1_VALUE (0x35U<<2)
#define MCU_REG_MRS_2_VALUE (0x36U<<2)
#define MCU_REG_MRS_3_VALUE (0x37U<<2)
#define MCU_REG_SCRUB_CTL (0x38U<<2)
#define MCU_REG_SCRUB_STATUS (0x39U<<2)
#define MCU_REG_SCRUB_ADDR_LO (0x3aU<<2)
#define MCU_REG_SCRUB_ADDR_HI (0x3bU<<2)
#define MCU_REG_SCRUB_START_ADDR_LO (0x3cU<<2)
#define MCU_REG_SCRUB_START_ADDR_HI (0x3dU<<2)
#define MCU_REG_SCRUB_END_ADDR_LO (0x3eU<<2)
#define MCU_REG_SCRUB_END_ADDR_HI (0x3fU<<2)
#define MCU_REG_RDRSPCOUNT (0x40U<<2)
#define MCU_REG_WRREQCOUNT (0x41U<<2)
#define MCU_REG_RMWREQCOUNT (0x42U<<2)
#define MCU_REG_MCUGECR (0x44U<<2)
#define MCU_REG_MCUGESR (0x45U<<2)
#define MCU_REG_MCUGESRA (0x46U<<2)
#define MCU_REG_FIFOESR (0x47U<<2)
#define MCU_REG_FIFOESRA (0x48U<<2)
#define MCU_REG_DFI_INIT_CTL (0x49U<<2)
#define MCU_REG_DFI_INIT_STS (0x4aU<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY0 (0x4eU<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY1 (0x4fU<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY2 (0x50U<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY3 (0x51U<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY4 (0x52U<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY5 (0x53U<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY6 (0x54U<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY7 (0x55U<<2)
#define MCU_REG_PHY_CSR_WLVL_DLY8 (0x56U<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY0 (0x57U<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY1 (0x58U<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY2 (0x59U<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY3 (0x5aU<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY4 (0x5bU<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY5 (0x5cU<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY6 (0x5dU<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY7 (0x5eU<<2)
#define MCU_REG_PHY_CSR_GLVL_DLY8 (0x60U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY0 (0x61U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY1 (0x62U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY2 (0x63U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY3 (0x64U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY4 (0x65U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY5 (0x66U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY6 (0x67U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY7 (0x68U<<2)
#define MCU_REG_PHY_CSR_RLVL_DLY8 (0x69U<<2)
#define MCU_REG_PHY_CSR_LVL_RANK_SEL (0x6aU<<2)
#define MCU_REG_PHY_CSR_LEVEL_LOAD (0x6bU<<2)
#define MCU_REG_RC0A (0x6cU<<2)
#define MCU_REG_RC0B (0x6dU<<2)
#define MCU_REG_RC1A (0x6eU<<2)
#define MCU_REG_RC1B (0x6fU<<2)
#define MCU_REG_RC2A (0x70U<<2)
#define MCU_REG_RC2B (0x71U<<2)
#define MCU_REG_RC3A (0x72U<<2)
#define MCU_REG_RC3B (0x73U<<2)
#define MCU_REG_WRLVLCTL (0x74U<<2)
#define MCU_REG_WRLVLDLY (0x75U<<2)
#define MCU_REG_RDLVLCTL (0x76U<<2)
#define MCU_REG_RDLVLDLY (0x77U<<2)
#define MCU_REG_RDGATECTL (0x78U<<2)
#define MCU_REG_TRNRSPSTS (0x79U<<2)
#define MCU_REG_CLKS200US (0x7aU<<2)
#define MCU_REG_CLKS500US (0x7bU<<2)
#define MCU_REG_CLKSTXPR (0x7cU<<2)
#define MCU_REG_CLKSTMRD (0x7dU<<2)
#define MCU_REG_CLKSTMOD (0x7eU<<2)
#define MCU_REG_CLKSTZQC (0x7fU<<2)
#define MCU_REG_CLKS6US (0x80U<<2)
#define MCU_REG_RDODTTABHI (0x81U<<2)
#define MCU_REG_RDODTTABLO (0x82U<<2)
#define MCU_REG_WRODTTABHI (0x83U<<2)
#define MCU_REG_WRODTTABLO (0x84U<<2)
#define MCU_REG_RDLVLRSP0 (0x85U<<2)
#define MCU_REG_RDLVLRSP1 (0x86U<<2)
#define MCU_REG_WRLVLRSP (0x87U<<2)
#define MCU_REG_RDLVLTMRS (0x88U<<2)
#define MCU_REG_WRLVLTMRS (0x89U<<2)
#define MCU_REG_WRLVLMRD (0x8aU<<2)
#define MCU_REG_BISTROWCOL (0x8bU<<2)
#define MCU_REG_BISTCTLSTS (0x8cU<<2)
#define MCU_REG_DDR_INIT_CTL (0x8dU<<2)
#define MCU_REG_DDR_INIT_SEQ (0x8eU<<2)
#define MCU_REG_DDR_INIT_STATUS (0x8fU<<2)
#define MCU_REG_PHY_TRNMODE (0x90U<<2)
#define MCU_REG_SWISS_ODTADDR (0x91U<<2)
#define MCU_REG_SWISS_OPCTL (0x92U<<2)
#define MCU_REG_SWISS_TRNCTL (0x93U<<2)
#define MCU_REG_SWISS_TRNWRLOAD (0x94U<<2)
#define MCU_REG_SWISS_TRNWRSTRB (0x95U<<2)
#define MCU_REG_SWISS_TRNRDLOAD (0x96U<<2)
#define MCU_REG_SWISS_RSTCKE (0x97U<<2)
#define MCU_REG_MCUSLEEPSTS (0xa0U<<2)
#define MCU_REG_MCUSLEEPCTL (0xa1U<<2)
#define MCU_REG_MCULSCTL (0xa2U<<2)
#define MCU_REG_MCUDSCTL (0xa3U<<2)
#define MCU_REG_MCUZQCCTL (0xa4U<<2)
#define MCU_REG_MCUZQCDLY (0xa5U<<2)
#define MCU_REG_RANK_SRXNODLL (0xa6U<<2)
#define MCU_REG_RANK_SRXDLL (0xa7U<<2)
#define MCU_REG_RANK_PDXNODLL (0xa8U<<2)
#define MCU_REG_RANK_PDXDLL (0xa9U<<2)
#define MCU_REG_RANK_RDPDE (0xaaU<<2)
#define MCU_REG_RANK_WRPDE (0xabU<<2)
#define MCU_REG_RANK_WRAPDE (0xacU<<2)
#define MCU_REG_RANK_CKEMIN (0xadU<<2)
#define MCU_REG_RANK_ODTL (0xaeU<<2)
#define MCU_REG_RAMSLEEPCTL (0xafU<<2)
#define MCU_REG_RAMSHUTDOWN (0xb0U<<2)
#define MCU_REG_RAMRWMCTL (0xb1U<<2)
#define MCU_REG_PHYLPCTL (0xb2U<<2)
#define MCU_REG_RAMPHYPWRSTS (0xb4U<<2)
#define MCU_REG_RANK_0_SIZE (0xc0U<<2)
#define MCU_REG_RANK_0_CONFIG (0xc1U<<2)
#define MCU_REG_RANK_0_BASE (0xc2U<<2)
#define MCU_REG_RANK_0_MASK (0xc3U<<2)
#define MCU_REG_RANK_0_HASH_SIZE (0xc4U<<2)
#define MCU_REG_MCUESRR0 (0xc5U<<2)
#define MCU_REG_MCUESRRA0 (0xc6U<<2)
#define MCU_REG_MCUEBLRR0 (0xc7U<<2)
#define MCU_REG_MCUERCRR0 (0xc8U<<2)
#define MCU_REG_MCUSBECNT0 (0xc9U<<2)
#define MCU_REG_RANK_1_SIZE (0xd0U<<2)
#define MCU_REG_RANK_1_CONFIG (0xd1U<<2)
#define MCU_REG_RANK_1_BASE (0xd2U<<2)
#define MCU_REG_RANK_1_MASK (0xd3U<<2)
#define MCU_REG_RANK_1_HASH_SIZE (0xd4U<<2)
#define MCU_REG_MCUESRR1 (0xd5U<<2)
#define MCU_REG_MCUESRRA1 (0xd6U<<2)
#define MCU_REG_MCUEBLRR1 (0xd7U<<2)
#define MCU_REG_MCUERCRR1 (0xd8U<<2)
#define MCU_REG_MCUSBECNT1 (0xd9U<<2)
#define MCU_REG_RANK_2_SIZE (0xe0U<<2)
#define MCU_REG_RANK_2_CONFIG (0xe1U<<2)
#define MCU_REG_RANK_2_BASE (0xe2U<<2)
#define MCU_REG_RANK_2_MASK (0xe3U<<2)
#define MCU_REG_RANK_2_HASH_SIZE (0xe4U<<2)
#define MCU_REG_MCUESRR2 (0xe5U<<2)
#define MCU_REG_MCUESRRA2 (0xe6U<<2)
#define MCU_REG_MCUEBLRR2 (0xe7U<<2)
#define MCU_REG_MCUERCRR2 (0xe8U<<2)
#define MCU_REG_MCUSBECNT2 (0xe9U<<2)
#define MCU_REG_RANK_3_SIZE (0xf0U<<2)
#define MCU_REG_RANK_3_CONFIG (0xf1U<<2)
#define MCU_REG_RANK_3_BASE (0xf2U<<2)
#define MCU_REG_RANK_3_MASK (0xf3U<<2)
#define MCU_REG_RANK_3_HASH_SIZE (0xf4U<<2)
#define MCU_REG_MCUESRR3 (0xf5U<<2)
#define MCU_REG_MCUESRRA3 (0xf6U<<2)
#define MCU_REG_MCUEBLRR3 (0xf7U<<2)
#define MCU_REG_MCUERCRR3 (0xf8U<<2)
#define MCU_REG_MCUSBECNT3 (0xf9U<<2)
#define MCU_REG_RANK_4_SIZE (0x100U<<2)
#define MCU_REG_RANK_4_CONFIG (0x101U<<2)
#define MCU_REG_RANK_4_BASE (0x102U<<2)
#define MCU_REG_RANK_4_MASK (0x103U<<2)
#define MCU_REG_RANK_4_HASH_SIZE (0x104U<<2)
#define MCU_REG_MCUESRR4 (0x105U<<2)
#define MCU_REG_MCUESRRA4 (0x106U<<2)
#define MCU_REG_MCUEBLRR4 (0x107U<<2)
#define MCU_REG_MCUERCRR4 (0x108U<<2)
#define MCU_REG_MCUSBECNT4 (0x109U<<2)
#define MCU_REG_RANK_5_SIZE (0x110U<<2)
#define MCU_REG_RANK_5_CONFIG (0x111U<<2)
#define MCU_REG_RANK_5_BASE (0x112U<<2)
#define MCU_REG_RANK_5_MASK (0x113U<<2)
#define MCU_REG_RANK_5_HASH_SIZE (0x114U<<2)
#define MCU_REG_MCUESRR5 (0x115U<<2)
#define MCU_REG_MCUESRRA5 (0x116U<<2)
#define MCU_REG_MCUEBLRR5 (0x117U<<2)
#define MCU_REG_MCUERCRR5 (0x118U<<2)
#define MCU_REG_MCUSBECNT5 (0x119U<<2)
#define MCU_REG_RANK_6_SIZE (0x120U<<2)
#define MCU_REG_RANK_6_CONFIG (0x121U<<2)
#define MCU_REG_RANK_6_BASE (0x122U<<2)
#define MCU_REG_RANK_6_MASK (0x123U<<2)
#define MCU_REG_RANK_6_HASH_SIZE (0x124U<<2)
#define MCU_REG_MCUESRR6 (0x125U<<2)
#define MCU_REG_MCUESRRA6 (0x126U<<2)
#define MCU_REG_MCUEBLRR6 (0x127U<<2)
#define MCU_REG_MCUERCRR6 (0x128U<<2)
#define MCU_REG_MCUSBECNT6 (0x129U<<2)
#define MCU_REG_RANK_7_SIZE (0x130U<<2)
#define MCU_REG_RANK_7_CONFIG (0x131U<<2)
#define MCU_REG_RANK_7_BASE (0x132U<<2)
#define MCU_REG_RANK_7_MASK (0x133U<<2)
#define MCU_REG_RANK_7_HASH_SIZE (0x134U<<2)
#define MCU_REG_MCUESRR7 (0x135U<<2)
#define MCU_REG_MCUESRRA7 (0x136U<<2)
#define MCU_REG_MCUEBLRR7 (0x137U<<2)
#define MCU_REG_MCUERCRR7 (0x138U<<2)
#define MCU_REG_MCUSBECNT7 (0x139U<<2)
#define MCU_REG_RDLVLRSPSTG1LO (0x140U<<2)
#define MCU_REG_RDLVLRSPSTG1HI (0x141U<<2)
#define MCU_REG_RDLVLRSPSTG2LO (0x142U<<2)
#define MCU_REG_RDLVLRSPSTG2HI (0x143U<<2)
#define MCU_REG_RDLVLRSPSTG3LO (0x144U<<2)
#define MCU_REG_RDLVLRSPSTG3HI (0x145U<<2)
#define MCU_REG_RDLVLRSPSTG4LO (0x146U<<2)
#define MCU_REG_RDLVLRSPSTG4HI (0x147U<<2)
#define MCU_REG_RDLVLRSPSTG5LO (0x148U<<2)
#define MCU_REG_RDLVLRSPSTG5HI (0x149U<<2)
#define MCU_REG_RDLVLRSPSTG6LO (0x14aU<<2)
#define MCU_REG_RDLVLRSPSTG6HI (0x14bU<<2)
#define MCU_REG_RDLVLRSPSTG7LO (0x14cU<<2)
#define MCU_REG_RDLVLRSPSTG7HI (0x14dU<<2)
#define MCU_REG_RDLVLRSPSTG8LO (0x14eU<<2)
#define MCU_REG_RDLVLRSPSTG8HI (0x14fU<<2)
#define MCU_REG_BISTWRDATA0 (0x150U<<2)
#define MCU_REG_BISTWRDATA1 (0x151U<<2)
#define MCU_REG_BISTWRDATA2 (0x152U<<2)
#define MCU_REG_BISTWRDATA3 (0x153U<<2)
#define MCU_REG_BISTWRDATA4 (0x154U<<2)
#define MCU_CPU_REG0 (0x0U<<2)
#define MCU_CPU_REG0_HPWEIGHT (0x0U<<2)
#define MCU_CPU_REG0_LPWEIGHT (0x1U<<2)
#define MCU_CPU_REG0_WPWEIGHT (0x2U<<2)
#define MCU_CPU_REG0_MAXINTERVAL (0x3U<<2)
#define MCU_CPU_REG0_QUANTUMSIZE (0x4U<<2)
#define MCU_CPU_REG0_AGETHRESH_HP (0x5U<<2)
#define MCU_CPU_REG0_AGETHRESH_LP (0x6U<<2)
#define MCU_CPU_REG0_AGETHRESH_WP (0x7U<<2)
#define MCU_CPU_REG0_PAGE_MODE (0x8U<<2)
#define MCU_CPU_REG0_HASH_CTL (0x9U<<2)
#define MCU_CPU_REG0_DIMM_ADDR_MIRROR_CTL (0xcU<<2)
#define MCU_CPU_REG0_FAST_CPU_ENABLE (0xdU<<2)
#define MCU_CPU_REG0_BANK_ACTPRE (0xeU<<2)
#define MCU_CPU_REG0_BANK_RDPRE (0xfU<<2)
#define MCU_CPU_REG0_BANK_WRPRE (0x10U<<2)
#define MCU_CPU_REG0_BANK_PREACT (0x11U<<2)
#define MCU_CPU_REG0_BANK_ACTACT (0x12U<<2)
#define MCU_CPU_REG0_BANK_RDAACT (0x13U<<2)
#define MCU_CPU_REG0_BANK_WRAACT (0x14U<<2)
#define MCU_CPU_REG0_BANK_ACTRD (0x15U<<2)
#define MCU_CPU_REG0_BANK_ACTWR (0x16U<<2)
#define MCU_CPU_REG0_BANK_ACTRDA (0x17U<<2)
#define MCU_CPU_REG0_BANK_ACTWRA (0x18U<<2)
#define MCU_CPU_REG0_RANK_ACTACT (0x19U<<2)
#define MCU_CPU_REG0_RANK_RDRD (0x1aU<<2)
#define MCU_CPU_REG0_RANK_WRWR (0x1bU<<2)
#define MCU_CPU_REG0_RANK_RDWR (0x1cU<<2)
#define MCU_CPU_REG0_RANK_WRRD (0x1dU<<2)
#define MCU_CPU_REG0_RANK_ACTFAW (0x1eU<<2)
#define MCU_CPU_REG0_SYS_RDRD (0x1fU<<2)
#define MCU_CPU_REG0_SYS_WRWR (0x20U<<2)
#define MCU_CPU_REG0_SYS_RDWR (0x21U<<2)
#define MCU_CPU_REG0_SYS_WRRD (0x22U<<2)
#define MCU_CPU_REG0_RANK_REFACTREF (0x23U<<2)
#define MCU_CPU_REG0_REFINT (0x24U<<2)
#define MCU_CPU_REG0_REFRESH_BURST_CNT (0x26U<<2)
#define MCU_CPU_REG0_T_WR_LATENCY (0x27U<<2)
#define MCU_CPU_REG0_T_RD_LATENCY (0x28U<<2)
#define MCU_CPU_REG0_BY4_MODE (0x29U<<2)
#define MCU_CPU_REG0_DDR_CLKDISABLE (0x2aU<<2)
#define MCU_CPU_REG0_UPDATECTL (0x2bU<<2)
#define MCU_CPU_REG0_PQDEBUGCTL (0x2cU<<2)
#define MCU_CPU_REG0_PQDEBUGDATAHI (0x2dU<<2)
#define MCU_CPU_REG0_PQDEBUGDATALO (0x2eU<<2)
#define MCU_CPU_REG0_MCUIDLESTS (0x2fU<<2)
#define MCU_CPU_REG0_MRS_0_VALUE (0x34U<<2)
#define MCU_CPU_REG0_MRS_1_VALUE (0x35U<<2)
#define MCU_CPU_REG0_MRS_2_VALUE (0x36U<<2)
#define MCU_CPU_REG0_MRS_3_VALUE (0x37U<<2)
#define MCU_CPU_REG0_SCRUB_CTL (0x38U<<2)
#define MCU_CPU_REG0_SCRUB_STATUS (0x39U<<2)
#define MCU_CPU_REG0_SCRUB_ADDR_LO (0x3aU<<2)
#define MCU_CPU_REG0_SCRUB_ADDR_HI (0x3bU<<2)
#define MCU_CPU_REG0_SCRUB_START_ADDR_LO (0x3cU<<2)
#define MCU_CPU_REG0_SCRUB_START_ADDR_HI (0x3dU<<2)
#define MCU_CPU_REG0_SCRUB_END_ADDR_LO (0x3eU<<2)
#define MCU_CPU_REG0_SCRUB_END_ADDR_HI (0x3fU<<2)
#define MCU_CPU_REG0_RDRSPCOUNT (0x40U<<2)
#define MCU_CPU_REG0_WRREQCOUNT (0x41U<<2)
#define MCU_CPU_REG0_RMWREQCOUNT (0x42U<<2)
#define MCU_CPU_REG0_MCUGECR (0x44U<<2)
#define MCU_CPU_REG0_MCUGESR (0x45U<<2)
#define MCU_CPU_REG0_MCUGESRA (0x46U<<2)
#define MCU_CPU_REG0_FIFOESR (0x47U<<2)
#define MCU_CPU_REG0_FIFOESRA (0x48U<<2)
#define MCU_CPU_REG0_DFI_INIT_CTL (0x49U<<2)
#define MCU_CPU_REG0_DFI_INIT_STS (0x4aU<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY0 (0x4eU<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY1 (0x4fU<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY2 (0x50U<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY3 (0x51U<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY4 (0x52U<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY5 (0x53U<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY6 (0x54U<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY7 (0x55U<<2)
#define MCU_CPU_REG0_PHY_CSR_WLVL_DLY8 (0x56U<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY0 (0x57U<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY1 (0x58U<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY2 (0x59U<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY3 (0x5aU<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY4 (0x5bU<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY5 (0x5cU<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY6 (0x5dU<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY7 (0x5eU<<2)
#define MCU_CPU_REG0_PHY_CSR_GLVL_DLY8 (0x60U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY0 (0x61U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY1 (0x62U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY2 (0x63U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY3 (0x64U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY4 (0x65U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY5 (0x66U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY6 (0x67U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY7 (0x68U<<2)
#define MCU_CPU_REG0_PHY_CSR_RLVL_DLY8 (0x69U<<2)
#define MCU_CPU_REG0_PHY_CSR_LVL_RANK_SEL (0x6aU<<2)
#define MCU_CPU_REG0_PHY_CSR_LEVEL_LOAD (0x6bU<<2)
#define MCU_CPU_REG0_RC0A (0x6cU<<2)
#define MCU_CPU_REG0_RC0B (0x6dU<<2)
#define MCU_CPU_REG0_RC1A (0x6eU<<2)
#define MCU_CPU_REG0_RC1B (0x6fU<<2)
#define MCU_CPU_REG0_RC2A (0x70U<<2)
#define MCU_CPU_REG0_RC2B (0x71U<<2)
#define MCU_CPU_REG0_RC3A (0x72U<<2)
#define MCU_CPU_REG0_RC3B (0x73U<<2)
#define MCU_CPU_REG0_WRLVLCTL (0x74U<<2)
#define MCU_CPU_REG0_WRLVLDLY (0x75U<<2)
#define MCU_CPU_REG0_RDLVLCTL (0x76U<<2)
#define MCU_CPU_REG0_RDLVLDLY (0x77U<<2)
#define MCU_CPU_REG0_RDGATECTL (0x78U<<2)
#define MCU_CPU_REG0_TRNRSPSTS (0x79U<<2)
#define MCU_CPU_REG0_CLKS200US (0x7aU<<2)
#define MCU_CPU_REG0_CLKS500US (0x7bU<<2)
#define MCU_CPU_REG0_CLKSTXPR (0x7cU<<2)
#define MCU_CPU_REG0_CLKSTMRD (0x7dU<<2)
#define MCU_CPU_REG0_CLKSTMOD (0x7eU<<2)
#define MCU_CPU_REG0_CLKSTZQC (0x7fU<<2)
#define MCU_CPU_REG0_CLKS6US (0x80U<<2)
#define MCU_CPU_REG0_RDODTTABHI (0x81U<<2)
#define MCU_CPU_REG0_RDODTTABLO (0x82U<<2)
#define MCU_CPU_REG0_WRODTTABHI (0x83U<<2)
#define MCU_CPU_REG0_WRODTTABLO (0x84U<<2)
#define MCU_CPU_REG0_RDLVLRSP0 (0x85U<<2)
#define MCU_CPU_REG0_RDLVLRSP1 (0x86U<<2)
#define MCU_CPU_REG0_WRLVLRSP (0x87U<<2)
#define MCU_CPU_REG0_RDLVLTMRS (0x88U<<2)
#define MCU_CPU_REG0_WRLVLTMRS (0x89U<<2)
#define MCU_CPU_REG0_WRLVLMRD (0x8aU<<2)
#define MCU_CPU_REG0_BISTROWCOL (0x8bU<<2)
#define MCU_CPU_REG0_BISTCTLSTS (0x8cU<<2)
#define MCU_CPU_REG0_DDR_INIT_CTL (0x8dU<<2)
#define MCU_CPU_REG0_DDR_INIT_SEQ (0x8eU<<2)
#define MCU_CPU_REG0_DDR_INIT_STATUS (0x8fU<<2)
#define MCU_CPU_REG0_PHY_TRNMODE (0x90U<<2)
#define MCU_CPU_REG0_SWISS_ODTADDR (0x91U<<2)
#define MCU_CPU_REG0_SWISS_OPCTL (0x92U<<2)
#define MCU_CPU_REG0_SWISS_TRNCTL (0x93U<<2)
#define MCU_CPU_REG0_SWISS_TRNWRLOAD (0x94U<<2)
#define MCU_CPU_REG0_SWISS_TRNWRSTRB (0x95U<<2)
#define MCU_CPU_REG0_SWISS_TRNRDLOAD (0x96U<<2)
#define MCU_CPU_REG0_SWISS_RSTCKE (0x97U<<2)
#define MCU_CPU_REG0_MCUSLEEPSTS (0xa0U<<2)
#define MCU_CPU_REG0_MCUSLEEPCTL (0xa1U<<2)
#define MCU_CPU_REG0_MCULSCTL (0xa2U<<2)
#define MCU_CPU_REG0_MCUDSCTL (0xa3U<<2)
#define MCU_CPU_REG0_MCUZQCCTL (0xa4U<<2)
#define MCU_CPU_REG0_MCUZQCDLY (0xa5U<<2)
#define MCU_CPU_REG0_RANK_SRXNODLL (0xa6U<<2)
#define MCU_CPU_REG0_RANK_SRXDLL (0xa7U<<2)
#define MCU_CPU_REG0_RANK_PDXNODLL (0xa8U<<2)
#define MCU_CPU_REG0_RANK_PDXDLL (0xa9U<<2)
#define MCU_CPU_REG0_RANK_RDPDE (0xaaU<<2)
#define MCU_CPU_REG0_RANK_WRPDE (0xabU<<2)
#define MCU_CPU_REG0_RANK_WRAPDE (0xacU<<2)
#define MCU_CPU_REG0_RANK_CKEMIN (0xadU<<2)
#define MCU_CPU_REG0_RANK_ODTL (0xaeU<<2)
#define MCU_CPU_REG0_RAMSLEEPCTL (0xafU<<2)
#define MCU_CPU_REG0_RAMSHUTDOWN (0xb0U<<2)
#define MCU_CPU_REG0_RAMRWMCTL (0xb1U<<2)
#define MCU_CPU_REG0_PHYLPCTL (0xb2U<<2)
#define MCU_CPU_REG0_RAMPHYPWRSTS (0xb4U<<2)
#define MCU_CPU_REG0_RANK_0_SIZE (0xc0U<<2)
#define MCU_CPU_REG0_RANK_0_CONFIG (0xc1U<<2)
#define MCU_CPU_REG0_RANK_0_BASE (0xc2U<<2)
#define MCU_CPU_REG0_RANK_0_MASK (0xc3U<<2)
#define MCU_CPU_REG0_RANK_0_HASH_SIZE (0xc4U<<2)
#define MCU_CPU_REG0_MCUESRR0 (0xc5U<<2)
#define MCU_CPU_REG0_MCUESRRA0 (0xc6U<<2)
#define MCU_CPU_REG0_MCUEBLRR0 (0xc7U<<2)
#define MCU_CPU_REG0_MCUERCRR0 (0xc8U<<2)
#define MCU_CPU_REG0_MCUSBECNT0 (0xc9U<<2)
#define MCU_CPU_REG0_RANK_1_SIZE (0xd0U<<2)
#define MCU_CPU_REG0_RANK_1_CONFIG (0xd1U<<2)
#define MCU_CPU_REG0_RANK_1_BASE (0xd2U<<2)
#define MCU_CPU_REG0_RANK_1_MASK (0xd3U<<2)
#define MCU_CPU_REG0_RANK_1_HASH_SIZE (0xd4U<<2)
#define MCU_CPU_REG0_MCUESRR1 (0xd5U<<2)
#define MCU_CPU_REG0_MCUESRRA1 (0xd6U<<2)
#define MCU_CPU_REG0_MCUEBLRR1 (0xd7U<<2)
#define MCU_CPU_REG0_MCUERCRR1 (0xd8U<<2)
#define MCU_CPU_REG0_MCUSBECNT1 (0xd9U<<2)
#define MCU_CPU_REG0_RANK_2_SIZE (0xe0U<<2)
#define MCU_CPU_REG0_RANK_2_CONFIG (0xe1U<<2)
#define MCU_CPU_REG0_RANK_2_BASE (0xe2U<<2)
#define MCU_CPU_REG0_RANK_2_MASK (0xe3U<<2)
#define MCU_CPU_REG0_RANK_2_HASH_SIZE (0xe4U<<2)
#define MCU_CPU_REG0_MCUESRR2 (0xe5U<<2)
#define MCU_CPU_REG0_MCUESRRA2 (0xe6U<<2)
#define MCU_CPU_REG0_MCUEBLRR2 (0xe7U<<2)
#define MCU_CPU_REG0_MCUERCRR2 (0xe8U<<2)
#define MCU_CPU_REG0_MCUSBECNT2 (0xe9U<<2)
#define MCU_CPU_REG0_RANK_3_SIZE (0xf0U<<2)
#define MCU_CPU_REG0_RANK_3_CONFIG (0xf1U<<2)
#define MCU_CPU_REG0_RANK_3_BASE (0xf2U<<2)
#define MCU_CPU_REG0_RANK_3_MASK (0xf3U<<2)
#define MCU_CPU_REG0_RANK_3_HASH_SIZE (0xf4U<<2)
#define MCU_CPU_REG0_MCUESRR3 (0xf5U<<2)
#define MCU_CPU_REG0_MCUESRRA3 (0xf6U<<2)
#define MCU_CPU_REG0_MCUEBLRR3 (0xf7U<<2)
#define MCU_CPU_REG0_MCUERCRR3 (0xf8U<<2)
#define MCU_CPU_REG0_MCUSBECNT3 (0xf9U<<2)
#define MCU_CPU_REG0_RANK_4_SIZE (0x100U<<2)
#define MCU_CPU_REG0_RANK_4_CONFIG (0x101U<<2)
#define MCU_CPU_REG0_RANK_4_BASE (0x102U<<2)
#define MCU_CPU_REG0_RANK_4_MASK (0x103U<<2)
#define MCU_CPU_REG0_RANK_4_HASH_SIZE (0x104U<<2)
#define MCU_CPU_REG0_MCUESRR4 (0x105U<<2)
#define MCU_CPU_REG0_MCUESRRA4 (0x106U<<2)
#define MCU_CPU_REG0_MCUEBLRR4 (0x107U<<2)
#define MCU_CPU_REG0_MCUERCRR4 (0x108U<<2)
#define MCU_CPU_REG0_MCUSBECNT4 (0x109U<<2)
#define MCU_CPU_REG0_RANK_5_SIZE (0x110U<<2)
#define MCU_CPU_REG0_RANK_5_CONFIG (0x111U<<2)
#define MCU_CPU_REG0_RANK_5_BASE (0x112U<<2)
#define MCU_CPU_REG0_RANK_5_MASK (0x113U<<2)
#define MCU_CPU_REG0_RANK_5_HASH_SIZE (0x114U<<2)
#define MCU_CPU_REG0_MCUESRR5 (0x115U<<2)
#define MCU_CPU_REG0_MCUESRRA5 (0x116U<<2)
#define MCU_CPU_REG0_MCUEBLRR5 (0x117U<<2)
#define MCU_CPU_REG0_MCUERCRR5 (0x118U<<2)
#define MCU_CPU_REG0_MCUSBECNT5 (0x119U<<2)
#define MCU_CPU_REG0_RANK_6_SIZE (0x120U<<2)
#define MCU_CPU_REG0_RANK_6_CONFIG (0x121U<<2)
#define MCU_CPU_REG0_RANK_6_BASE (0x122U<<2)
#define MCU_CPU_REG0_RANK_6_MASK (0x123U<<2)
#define MCU_CPU_REG0_RANK_6_HASH_SIZE (0x124U<<2)
#define MCU_CPU_REG0_MCUESRR6 (0x125U<<2)
#define MCU_CPU_REG0_MCUESRRA6 (0x126U<<2)
#define MCU_CPU_REG0_MCUEBLRR6 (0x127U<<2)
#define MCU_CPU_REG0_MCUERCRR6 (0x128U<<2)
#define MCU_CPU_REG0_MCUSBECNT6 (0x129U<<2)
#define MCU_CPU_REG0_RANK_7_SIZE (0x130U<<2)
#define MCU_CPU_REG0_RANK_7_CONFIG (0x131U<<2)
#define MCU_CPU_REG0_RANK_7_BASE (0x132U<<2)
#define MCU_CPU_REG0_RANK_7_MASK (0x133U<<2)
#define MCU_CPU_REG0_RANK_7_HASH_SIZE (0x134U<<2)
#define MCU_CPU_REG0_MCUESRR7 (0x135U<<2)
#define MCU_CPU_REG0_MCUESRRA7 (0x136U<<2)
#define MCU_CPU_REG0_MCUEBLRR7 (0x137U<<2)
#define MCU_CPU_REG0_MCUERCRR7 (0x138U<<2)
#define MCU_CPU_REG0_MCUSBECNT7 (0x139U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG1LO (0x140U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG1HI (0x141U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG2LO (0x142U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG2HI (0x143U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG3LO (0x144U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG3HI (0x145U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG4LO (0x146U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG4HI (0x147U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG5LO (0x148U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG5HI (0x149U<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG6LO (0x14aU<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG6HI (0x14bU<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG7LO (0x14cU<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG7HI (0x14dU<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG8LO (0x14eU<<2)
#define MCU_CPU_REG0_RDLVLRSPSTG8HI (0x14fU<<2)
#define MCU_CPU_REG0_BISTWRDATA0 (0x150U<<2)
#define MCU_CPU_REG0_BISTWRDATA1 (0x151U<<2)
#define MCU_CPU_REG0_BISTWRDATA2 (0x152U<<2)
#define MCU_CPU_REG0_BISTWRDATA3 (0x153U<<2)
#define MCU_CPU_REG0_BISTWRDATA4 (0x154U<<2)
#define MCU_HPWEIGHT_RESERVED_RNG 0xaU
#define MCU_HPWEIGHT_WEIGHT_VALUE_RNG 0x0U
#define MCU_LPWEIGHT_RESERVED_RNG 0xaU
#define MCU_LPWEIGHT_WEIGHT_VALUE_RNG 0x0U
#define MCU_WPWEIGHT_RESERVED_RNG  0xaU
#define MCU_WPWEIGHT_WEIGHT_VALUE_RNG 0x0U
#define MCU_MAXINTERVAL_RESERVED_RNG 0x10U
#define MCU_MAXINTERVAL_INTERVAL_VALUE_RNG 0x0U
#define MCU_QUANTUMSIZE_RESERVED_RNG  0x4U
#define MCU_QUANTUMSIZE_SIZE_VALUE_RNG 0x0
#define MCU_AGETHRESH_HP_RESERVED_RNG 0x8
#define MCU_AGETHRESH_HP_THRESHOLD_VALUE_RNG 0x0
#define MCU_AGETHRESH_LP_RESERVED_RNG 0x8
#define MCU_AGETHRESH_LP_THRESHOLD_VALUE_RNG 0x0
#define MCU_AGETHRESH_WP_RESERVED_RNG 0x8
#define MCU_AGETHRESH_WP_THRESHOLD_VALUE_RNG 0x0
#define MCU_PAGE_MODE_RESERVED_RNG 0x1
#define MCU_PAGE_MODE_OPEN_PAGE_RNG 0x0
#define MCU_HASH_CTL_RESERVED_RNG 0x1
#define MCU_HASH_CTL_ENABLE_RNG 0x0
#define MCU_DIMM_ADDR_MIRROR_CTL_RESERVED_RNG 0x8
#define MCU_DIMM_ADDR_MIRROR_CTL_ENABLE_RNG 0x0
#define MCU_FAST_CPU_ENABLE_RESERVED_RNG 0x1
#define MCU_FAST_CPU_ENABLE_ENABLE_RNG 0x0
#define MCU_BANK_ACTPRE_RESERVED_RNG 0x6
#define MCU_BANK_ACTPRE_TIMER_VALUE_RNG 0x0
#define MCU_BANK_RDPRE_RESERVED_RNG 0x6
#define MCU_BANK_RDPRE_TIMER_VALUE_RNG 0x0
#define MCU_BANK_WRPRE_RESERVED_RNG 0x6
#define MCU_BANK_WRPRE_TIMER_VALUE_RNG 0x0
#define MCU_BANK_PREACT_RESERVED_RNG 0x6
#define MCU_BANK_PREACT_TIMER_VALUE_RNG 0x0
#define MCU_BANK_ACTACT_RESERVED_RNG 0x6
#define MCU_BANK_ACTACT_TIMER_VALUE_RNG 0x0
#define MCU_BANK_RDAACT_RESERVED_RNG 0x6
#define MCU_BANK_RDAACT_TIMER_VALUE_RNG 0x0
#define MCU_BANK_WRAACT_RESERVED_RNG 0x6
#define MCU_BANK_WRAACT_TIMER_VALUE_RNG 0x0
#define MCU_BANK_ACTRD_RESERVED_RNG 0x6
#define MCU_BANK_ACTRD_TIMER_VALUE_RNG 0x0
#define MCU_BANK_ACTWR_RESERVED_RNG 0x6
#define MCU_BANK_ACTWR_TIMER_VALUE_RNG 0x0
#define MCU_BANK_ACTRDA_RESERVED_RNG 0x6
#define MCU_BANK_ACTRDA_TIMER_VALUE_RNG 0x0
#define MCU_BANK_ACTWRA_RESERVED_RNG 0x6
#define MCU_BANK_ACTWRA_TIMER_VALUE_RNG 0x0
#define MCU_RANK_ACTACT_RESERVED_RNG 0x6
#define MCU_RANK_ACTACT_TIMER_VALUE_RNG 0x0
#define MCU_RANK_RDRD_RESERVED_RNG 0x6
#define MCU_RANK_RDRD_TIMER_VALUE_RNG 0x0
#define MCU_RANK_WRWR_RESERVED_RNG 0x6
#define MCU_RANK_WRWR_TIMER_VALUE_RNG 0x0
#define MCU_RANK_RDWR_RESERVED_RNG 0x6
#define MCU_RANK_RDWR_TIMER_VALUE_RNG 0x0
#define MCU_RANK_WRRD_RESERVED_RNG 0x6
#define MCU_RANK_WRRD_TIMER_VALUE_RNG 0x0
#define MCU_RANK_ACTFAW_RESERVED_RNG 0x6
#define MCU_RANK_ACTFAW_TIMER_VALUE_RNG 0x0
#define MCU_SYS_RDRD_RESERVED_RNG 0x6
#define MCU_SYS_RDRD_TIMER_VALUE_RNG 0x0
#define MCU_SYS_WRWR_RESERVED_RNG 0x6
#define MCU_SYS_WRWR_TIMER_VALUE_RNG 0x0
#define MCU_SYS_RDWR_RESERVED_RNG 0x6
#define MCU_SYS_RDWR_TIMER_VALUE_RNG 0x0
#define MCU_SYS_WRRD_RESERVED_RNG 0x6
#define MCU_SYS_WRRD_TIMER_VALUE_RNG 0x0
#define MCU_RANK_REFACTREF_RESERVED_RNG 0x9
#define MCU_RANK_REFACTREF_TIMER_VALUE_RNG 0x0
#define MCU_REFINT_RESERVED_RNG 0x11
#define MCU_REFINT_TIMER_VALUE_RNG 0x0
#define MCU_REFRESH_BURST_CNT_RESERVED_RNG 0x3
#define MCU_REFRESH_BURST_CNT_COUNT_VALUE_RNG 0x0
#define MCU_T_WR_LATENCY_RESERVED_RNG 0x4
#define MCU_T_WR_LATENCY_LATENCY_VALUE_RNG 0x0
#define MCU_T_RD_LATENCY_RESERVED_RNG 0x5
#define MCU_T_RD_LATENCY_LATENCY_VALUE_RNG 0x0
#define MCU_BY4_MODE_RESERVED_RNG 0x1
#define MCU_BY4_MODE_ENABLED_RNG 0x0
#define MCU_DDR_CLKDISABLE_RESERVED_RNG 0x8
#define MCU_DDR_CLKDISABLE_DISABLE_RNG 0x0
#define MCU_UPDATECTL_RESERVED_RNG 0x8
#define MCU_UPDATECTL_WAITCNT_RNG 0x0
#define MCU_PQDEBUGCTL_CAPTURE_RNG 0x1f
#define MCU_PQDEBUGCTL_ENABLE_RT_DEBUG_RNG 0x1e
#define MCU_PQDEBUGCTL_RT_I_AM_SRC_RNG 0x1d
#define MCU_PQDEBUGCTL_INVERT_RCV_CLK_RNG 0x1c
#define MCU_PQDEBUGCTL_RESERVED_RNG 0xc
#define MCU_PQDEBUGCTL_RT_DBG_WRD_SEL_RNG 0xb
#define MCU_PQDEBUGCTL_RT_DBG_HW_SEL_RNG 0xa
#define MCU_PQDEBUGCTL_DEBUG_SRC_SEL_RNG 0x8
#define MCU_PQDEBUGCTL_QUEUE_SELECT_RNG 0x5
#define MCU_PQDEBUGCTL_ENTRY_SELECT_RNG 0x0
#define MCU_PQDEBUGDATAHI_PQ_DBG_DATA_HI_RNG 0x0
#define MCU_PQDEBUGDATALO_PQ_DBG_DATA_LO_RNG 0x0
#define MCU_MCUIDLESTS_MCUQUIESCED_RNG 0x1f
#define MCU_MCUIDLESTS_RESERVED_RNG 0x1b
#define MCU_MCUIDLESTS_MCUREQVALID_RNG 0x1a
#define MCU_MCUIDLESTS_INPUTBUFFEREMPTY_RNG 0x19
#define MCU_MCUIDLESTS_WRMEMEMPTY_RNG 0x18
#define MCU_MCUIDLESTS_BQRANKIDLE_RNG 0x10
#define MCU_MCUIDLESTS_CQRANKIDLE_RNG 0x8
#define MCU_MCUIDLESTS_PQRANKIDLE_RNG 0x0
#define MCU_MRS_0_VALUE_RESERVED_RNG 0x10
#define MCU_MRS_0_VALUE_REGISTER_VALUE0_RNG 0xd
#define MCU_MRS_0_VALUE_PPD_RNG 0xc
#define MCU_MRS_0_VALUE_WR_RNG 0x9
#define MCU_MRS_0_VALUE_DLL_RESET_RNG 0x8
#define MCU_MRS_0_VALUE_TEST_RNG 0x7
#define MCU_MRS_0_VALUE_CL_RNG 0x4
#define MCU_MRS_0_VALUE_RBT_RNG 0x3
#define MCU_MRS_0_VALUE_CL2_RNG 0x2
#define MCU_MRS_0_VALUE_BL_RNG 0x0
#define MCU_MRS_1_VALUE_RESERVED_RNG 0x10
#define MCU_MRS_1_VALUE_REGISTER_VALUE_RNG 0x0
#define MCU_MRS_2_VALUE_RESERVED_RNG 0x10
#define MCU_MRS_2_VALUE_REGISTER_VALUE1_RNG 0x6
#define MCU_MRS_2_VALUE_CWL_RNG 0x3
#define MCU_MRS_2_VALUE_REGISTER_VALUE0_RNG 0x0
#define MCU_MRS_3_VALUE_RESERVED_RNG 0x10
#define MCU_MRS_3_VALUE_REGISTER_VALUE_RNG 0x0
#define MCU_SCRUB_CTL_RESERVED_RNG 0x8
#define MCU_SCRUB_CTL_INTERVAL_VALUE_RNG 0x4
#define MCU_SCRUB_CTL_RESERVED1_RNG 0x2
#define MCU_SCRUB_CTL_DMD_SCRUB_ENABLE_RNG 0x1
#define MCU_SCRUB_CTL_ENABLE_RNG 0x0
#define MCU_SCRUB_STATUS_RESERVED_RNG 0x1
#define MCU_SCRUB_STATUS_SCRUB_COMPLETE_RNG 0x0
#define MCU_SCRUB_ADDR_LO_ADDRESS_VALUE_RNG 0x0
#define MCU_SCRUB_ADDR_HI_RESERVED_RNG 0x4
#define MCU_SCRUB_ADDR_HI_ADDRESS_VALUE_RNG 0x0
#define MCU_SCRUB_START_ADDR_LO_ADDRESS_VALUE_RNG 0x0
#define MCU_SCRUB_START_ADDR_HI_RESERVED_RNG 0x4
#define MCU_SCRUB_START_ADDR_HI_ADDRESS_VALUE_RNG 0x0
#define MCU_SCRUB_END_ADDR_LO_ADDRESS_VALUE_RNG 0x0
#define MCU_SCRUB_END_ADDR_HI_RESERVED_RNG 0x4
#define MCU_SCRUB_END_ADDR_HI_ADDRESS_VALUE_RNG 0x0
#define MCU_RDRSPCOUNT_RESERVED_RNG 0x1c
#define MCU_RDRSPCOUNT_VALUE_RNG 0x0
#define MCU_WRREQCOUNT_RESERVED_RNG 0x1c
#define MCU_WRREQCOUNT_VALUE_RNG 0x0
#define MCU_RMWREQCOUNT_RESERVED_RNG 0x1c
#define MCU_RMWREQCOUNT_VALUE_RNG 0x0
#define MCU_MCUGECR_CORRTHRESH_RNG 0x10
#define MCU_MCUGECR_RESERVED_RNG 0xb
#define MCU_MCUGECR_MCUOVUNERREN_RNG 0xa
#define MCU_MCUGECR_MCUADDRERREN_RNG 0x9
#define MCU_MCUGECR_PHYTRNGERREN_RNG 0x8
#define MCU_MCUGECR_ECCERREN_RNG 0x7
#define MCU_MCUGECR_RESERVED1_RNG 0x4
#define MCU_MCUGECR_PHYPERREN_RNG 0x3
#define MCU_MCUGECR_CINTREN_RNG 0x2
#define MCU_MCUGECR_BACKUCINTREN_RNG 0x1
#define MCU_MCUGECR_DEMANDUCINTREN_RNG 0x0
#define MCU_MCUGESR_RESERVED_RNG 0xc
#define MCU_MCUGESR_PHYTRAININGERR_RNG 0xb
#define MCU_MCUGESR_WRLVLTIMEOUTERR_RNG 0xa
#define MCU_MCUGESR_RDGATETIMEOUTERR_RNG 0x9
#define MCU_MCUGESR_RDLVLTIMEOUTERR_RNG 0x8
#define MCU_MCUGESR_ADDRNOMATCHERR_RNG 0x7
#define MCU_MCUGESR_ADDRMULTIMATCHERR_RNG 0x6
#define MCU_MCUGESR_RESERVED1_RNG 0x4
#define MCU_MCUGESR_PHYPERR_RNG 0x3
#define MCU_MCUGESR_CERR_RNG 0x2
#define MCU_MCUGESR_BACKUCERR_RNG 0x1
#define MCU_MCUGESR_DEMANDUCERR_RNG 0x0
#define MCU_MCUGESRA_RESERVED_RNG 0xc
#define MCU_MCUGESRA_PHYTRAININGERR_RNG 0xb
#define MCU_MCUGESRA_WRLVLTIMEOUTERR_RNG 0xa
#define MCU_MCUGESRA_RDGATETIMEOUTERR_RNG 0x9
#define MCU_MCUGESRA_RDLVLTIMEOUTERR_RNG 0x8
#define MCU_MCUGESRA_ADDRNOMATCHERR_RNG 0x7
#define MCU_MCUGESRA_ADDRMULTIMATCHERR_RNG 0x6
#define MCU_MCUGESRA_RESERVED1_RNG 0x4
#define MCU_MCUGESRA_PHYPERR_RNG 0x3
#define MCU_MCUGESRA_CERR_RNG 0x2
#define MCU_MCUGESRA_BACKUCERR_RNG 0x1
#define MCU_MCUGESRA_DEMANDUCERR_RNG 0x0
#define MCU_FIFOESR_RESERVED_RNG 0x18
#define MCU_FIFOESR_RDDATA_FIFOCREDIT_UNDERFLOW_RNG 0x17
#define MCU_FIFOESR_RDDATA_FIFOCREDIT_OVERFLOW_RNG 0x16
#define MCU_FIFOESR_WRMEM_UNDERFLOW_RNG 0x15
#define MCU_FIFOESR_WRMEM_OVERFLOW_RNG 0x14
#define MCU_FIFOESR_ID_FIFO_UNDERFLOW_RNG 0x13
#define MCU_FIFOESR_ID_FIFO_OVERFLOW_RNG 0x12
#define MCU_FIFOESR_RDDATACREDIT_UNDERFLOW_RNG 0x11
#define MCU_FIFOESR_RDDATACREDIT_OVERFLOW_RNG 0x10
#define MCU_FIFOESR_WRPROC_UNDERFLOW_RNG 0xf
#define MCU_FIFOESR_WRPROC_OVERFLOW_RNG 0xe
#define MCU_FIFOESR_RDPROC_UNDERFLOW_RNG 0xd
#define MCU_FIFOESR_RDPROC_OVERFLOW_RNG 0xc
#define MCU_FIFOESR_CQUEUE_UNDERFLOW_RNG 0xb
#define MCU_FIFOESR_CQUEUE_OVERFLOW_RNG 0xa
#define MCU_FIFOESR_SEQWR_UNDERFLOW_RNG 0x9
#define MCU_FIFOESR_SEQWR_OVERFLOW_RNG 0x8
#define MCU_FIFOESR_SEQRD1_UNDERFLOW_RNG 0x7
#define MCU_FIFOESR_SEQRD1_OVERFLOW_RNG 0x6
#define MCU_FIFOESR_SEQRD0_UNDERFLOW_RNG 0x5
#define MCU_FIFOESR_SEQRD0_OVERFLOW_RNG 0x4
#define MCU_FIFOESR_PQUEUE_UNDERFLOW_RNG 0x3
#define MCU_FIFOESR_PQUEUE_OVERFLOW_RNG 0x2
#define MCU_FIFOESR_INBUFFER_UNDERFLOW_RNG 0x1
#define MCU_FIFOESR_INBUFFER_OVERFLOW_RNG 0x0
#define MCU_FIFOESRA_RESERVED_RNG 0x18
#define MCU_FIFOESRA_RDDATA_FIFOCREDIT_UNDERFLOW_RNG 0x17
#define MCU_FIFOESRA_RDDATA_FIFOCREDIT_OVERFLOW_RNG 0x16
#define MCU_FIFOESRA_WRMEM_UNDERFLOW_RNG 0x15
#define MCU_FIFOESRA_WRMEM_OVERFLOW_RNG 0x14
#define MCU_FIFOESRA_ID_FIFO_UNDERFLOW_RNG 0x13
#define MCU_FIFOESRA_ID_FIFO_OVERFLOW_RNG 0x12
#define MCU_FIFOESRA_RDDATACREDIT_UNDERFLOW_RNG 0x11
#define MCU_FIFOESRA_RDDATACREDIT_OVERFLOW_RNG 0x10
#define MCU_FIFOESRA_WRPROC_UNDERFLOW_RNG 0xf
#define MCU_FIFOESRA_WRPROC_OVERFLOW_RNG 0xe
#define MCU_FIFOESRA_RDPROC_UNDERFLOW_RNG 0xd
#define MCU_FIFOESRA_RDPROC_OVERFLOW_RNG 0xc
#define MCU_FIFOESRA_CQUEUE_UNDERFLOW_RNG 0xb
#define MCU_FIFOESRA_CQUEUE_OVERFLOW_RNG 0xa
#define MCU_FIFOESRA_SEQWR_UNDERFLOW_RNG 0x9
#define MCU_FIFOESRA_SEQWR_OVERFLOW_RNG 0x8
#define MCU_FIFOESRA_SEQRD1_UNDERFLOW_RNG 0x7
#define MCU_FIFOESRA_SEQRD1_OVERFLOW_RNG 0x6
#define MCU_FIFOESRA_SEQRD0_UNDERFLOW_RNG 0x5
#define MCU_FIFOESRA_SEQRD0_OVERFLOW_RNG 0x4
#define MCU_FIFOESRA_PQUEUE_UNDERFLOW_RNG 0x3
#define MCU_FIFOESRA_PQUEUE_OVERFLOW_RNG 0x2
#define MCU_FIFOESRA_INBUFFER_UNDERFLOW_RNG 0x1
#define MCU_FIFOESRA_INBUFFER_OVERFLOW_RNG 0x0
#define MCU_DFI_INIT_CTL_RESERVED_RNG 0x3
#define MCU_DFI_INIT_CTL_DFI_RESET_RNG 0x2
#define MCU_DFI_INIT_CTL_RESERVED1_RNG 0x1
#define MCU_DFI_INIT_CTL_INIT_START_RNG 0x0
#define MCU_DFI_INIT_STS_RESERVED_RNG 0x1
#define MCU_DFI_INIT_STS_INIT_COMPLETE_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY0_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY0_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY0_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY1_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY1_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY1_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY2_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY2_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY2_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY3_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY3_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY3_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY4_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY4_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY4_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY5_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY5_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY5_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY6_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY6_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY6_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY7_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY7_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY7_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_WLVL_DLY8_RESERVED_RNG 0x18
#define MCU_PHY_CSR_WLVL_DLY8_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_WLVL_DLY8_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY0_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY0_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY0_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY1_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY1_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY1_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY2_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY2_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY2_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY3_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY3_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY3_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY4_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY4_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY4_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY5_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY5_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY5_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY6_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY6_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY6_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY7_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY7_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY7_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_GLVL_DLY8_RESERVED_RNG 0x18
#define MCU_PHY_CSR_GLVL_DLY8_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_GLVL_DLY8_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY0_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY0_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY0_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY1_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY1_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY1_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY2_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY2_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY2_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY3_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY3_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY3_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY4_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY4_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY4_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY5_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY5_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY5_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY6_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY6_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY6_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY7_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY7_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY7_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_RLVL_DLY8_RESERVED_RNG 0x18
#define MCU_PHY_CSR_RLVL_DLY8_DLY_N1_RNG 0xc
#define MCU_PHY_CSR_RLVL_DLY8_DLY_N0_RNG 0x0
#define MCU_PHY_CSR_LVL_RANK_SEL_RESERVED_RNG 0x3
#define MCU_PHY_CSR_LVL_RANK_SEL_RANK_NUM_RNG 0x0
#define MCU_PHY_CSR_LEVEL_LOAD_RESERVED_RNG 0x1
#define MCU_PHY_CSR_LEVEL_LOAD_PHY_RDLVL_LOAD_RNG 0x0
#define MCU_RC0A_CALIBRATION_VALUE_07_RNG 0x1c
#define MCU_RC0A_CALIBRATION_VALUE_06_RNG 0x18
#define MCU_RC0A_CALIBRATION_VALUE_05_RNG 0x14
#define MCU_RC0A_CALIBRATION_VALUE_04_RNG 0x10
#define MCU_RC0A_CALIBRATION_VALUE_03_RNG 0xc
#define MCU_RC0A_CALIBRATION_VALUE_02_RNG 0x8
#define MCU_RC0A_CALIBRATION_VALUE_01_RNG 0x4
#define MCU_RC0A_CALIBRATION_VALUE_00_RNG 0x0
#define MCU_RC0B_RESERVED_RNG 0x10
#define MCU_RC0B_CALIBRATION_VALUE_11_RNG 0xc
#define MCU_RC0B_CALIBRATION_VALUE_10_RNG 0x8
#define MCU_RC0B_CALIBRATION_VALUE_09_RNG 0x4
#define MCU_RC0B_CALIBRATION_VALUE_08_RNG 0x0
#define MCU_RC1A_CALIBRATION_VALUE_07_RNG 0x1c
#define MCU_RC1A_CALIBRATION_VALUE_06_RNG 0x18
#define MCU_RC1A_CALIBRATION_VALUE_05_RNG 0x14
#define MCU_RC1A_CALIBRATION_VALUE_04_RNG 0x10
#define MCU_RC1A_CALIBRATION_VALUE_03_RNG 0xc
#define MCU_RC1A_CALIBRATION_VALUE_02_RNG 0x8
#define MCU_RC1A_CALIBRATION_VALUE_01_RNG 0x4
#define MCU_RC1A_CALIBRATION_VALUE_00_RNG 0x0
#define MCU_RC1B_RESERVED_RNG 0x10
#define MCU_RC1B_CALIBRATION_VALUE_11_RNG 0xc
#define MCU_RC1B_CALIBRATION_VALUE_10_RNG 0x8
#define MCU_RC1B_CALIBRATION_VALUE_09_RNG 0x4
#define MCU_RC1B_CALIBRATION_VALUE_08_RNG 0x0
#define MCU_RC2A_CALIBRATION_VALUE_07_RNG 0x1c
#define MCU_RC2A_CALIBRATION_VALUE_06_RNG 0x18
#define MCU_RC2A_CALIBRATION_VALUE_05_RNG 0x14
#define MCU_RC2A_CALIBRATION_VALUE_04_RNG 0x10
#define MCU_RC2A_CALIBRATION_VALUE_03_RNG 0xc
#define MCU_RC2A_CALIBRATION_VALUE_02_RNG 0x8
#define MCU_RC2A_CALIBRATION_VALUE_01_RNG 0x4
#define MCU_RC2A_CALIBRATION_VALUE_00_RNG 0x0
#define MCU_RC2B_RESERVED_RNG 0x10
#define MCU_RC2B_CALIBRATION_VALUE_11_RNG 0xc
#define MCU_RC2B_CALIBRATION_VALUE_10_RNG 0x8
#define MCU_RC2B_CALIBRATION_VALUE_09_RNG 0x4
#define MCU_RC2B_CALIBRATION_VALUE_08_RNG 0x0
#define MCU_RC3A_CALIBRATION_VALUE_07_RNG 0x1c
#define MCU_RC3A_CALIBRATION_VALUE_06_RNG 0x18
#define MCU_RC3A_CALIBRATION_VALUE_05_RNG 0x14
#define MCU_RC3A_CALIBRATION_VALUE_04_RNG 0x10
#define MCU_RC3A_CALIBRATION_VALUE_03_RNG 0xc
#define MCU_RC3A_CALIBRATION_VALUE_02_RNG 0x8
#define MCU_RC3A_CALIBRATION_VALUE_01_RNG 0x4
#define MCU_RC3A_CALIBRATION_VALUE_00_RNG 0x0
#define MCU_RC3B_RESERVED_RNG 0x10
#define MCU_RC3B_CALIBRATION_VALUE_11_RNG 0xc
#define MCU_RC3B_CALIBRATION_VALUE_10_RNG 0x8
#define MCU_RC3B_CALIBRATION_VALUE_09_RNG 0x4
#define MCU_RC3B_CALIBRATION_VALUE_08_RNG 0x0
#define MCU_WRLVLCTL_RESERVED_RNG 0x4
#define MCU_WRLVLCTL_WRLVL_EN_RNG 0x3
#define MCU_WRLVLCTL_WRLVLALLDONE_RNG 0x2
#define MCU_WRLVLCTL_DOWRLVL_RNG 0x1
#define MCU_WRLVLCTL_WRLVLENDRANK_RNG 0x0
#define MCU_WRLVLDLY_WRRESPLAT_RNG 0x18
#define MCU_WRLVLDLY_MR2WRLVLEN_RNG 0x10
#define MCU_WRLVLDLY_WRLVLDLL_RNG 0x8
#define MCU_WRLVLDLY_WRLVLEN2LOAD_RNG 0x0
#define MCU_RDLVLCTL_RESERVED_RNG 0x16
#define MCU_RDLVLCTL_SAMPLE_DELAY_RNG 0x10
#define MCU_RDLVLCTL_RESERVED1_RNG 0x9
#define MCU_RDLVLCTL_RDFIFO_RESET_RNG 0x8
#define MCU_RDLVLCTL_RESERVED2_RNG 0x4
#define MCU_RDLVLCTL_RDLVL_EN_RNG 0x3
#define MCU_RDLVLCTL_RDLVLALLDONE_RNG 0x2
#define MCU_RDLVLCTL_DORDLVL_RNG 0x1
#define MCU_RDLVLCTL_RDLVLENDRANK_RNG 0x0
#define MCU_RDLVLDLY_RDRESPLAT_RNG 0x18
#define MCU_RDLVLDLY_MR2RDLVLEN_RNG 0x10
#define MCU_RDLVLDLY_RDLVLDLL_RNG 0x8
#define MCU_RDLVLDLY_RDLVLENLOAD_RNG 0x0
#define MCU_RDGATECTL_RESERVED_RNG 0x4
#define MCU_RDGATECTL_RDGATE_EN_RNG 0x3
#define MCU_RDGATECTL_RDGATEALLDONE_RNG 0x2
#define MCU_RDGATECTL_DORDGATE_RNG 0x1
#define MCU_RDGATECTL_RDGATEENDRANK_RNG 0x0
#define MCU_TRNRSPSTS_RESERVED_RNG 0x3
#define MCU_TRNRSPSTS_WRLVLRSPRDY_RNG 0x2
#define MCU_TRNRSPSTS_RDLVLRSPRDY_RNG 0x1
#define MCU_TRNRSPSTS_RDGATERSPRDY_RNG 0x0
#define MCU_CLKS200US_RESERVED_RNG 0x13
#define MCU_CLKS200US_TIMER_VALUE_RNG 0x0
#define MCU_CLKS500US_RESERVED_RNG 0x14
#define MCU_CLKS500US_TIMER_VALUE_RNG 0x0
#define MCU_CLKSTXPR_RESERVED_RNG 0x9
#define MCU_CLKSTXPR_TIMER_VALUE_RNG 0x0
#define MCU_CLKSTMRD_RESERVED_RNG 0x4
#define MCU_CLKSTMRD_TIMER_VALUE_RNG 0x0
#define MCU_CLKSTMOD_RESERVED_RNG 0x8
#define MCU_CLKSTMOD_TIMER_VALUE_RNG 0x0
#define MCU_CLKSTZQC_RESERVED_RNG 0x18
#define MCU_CLKSTZQC_ZQS_RNG 0xc
#define MCU_CLKSTZQC_ZQL_RNG 0x0
#define MCU_CLKS6US_RESERVED_RNG 0xd
#define MCU_CLKS6US_TIMER_VALUE_RNG 0x0
#define MCU_RDODTTABHI_RDODTRANK7_RNG 0x18
#define MCU_RDODTTABHI_RDODTRANK6_RNG 0x10
#define MCU_RDODTTABHI_RDODTRANK5_RNG 0x8
#define MCU_RDODTTABHI_RDODTRANK4_RNG 0x0
#define MCU_RDODTTABLO_RDODTRANK3_RNG 0x18
#define MCU_RDODTTABLO_RDODTRANK2_RNG 0x10
#define MCU_RDODTTABLO_RDODTRANK1_RNG 0x8
#define MCU_RDODTTABLO_RDODTRANK0_RNG 0x0
#define MCU_WRODTTABHI_WRODTRANK7_RNG 0x18
#define MCU_WRODTTABHI_WRODTRANK6_RNG 0x10
#define MCU_WRODTTABHI_WRODTRANK5_RNG 0x8
#define MCU_WRODTTABHI_WRODTRANK4_RNG 0x0
#define MCU_WRODTTABLO_WRODTRANK3_RNG 0x18
#define MCU_WRODTTABLO_WRODTRANK2_RNG 0x10
#define MCU_WRODTTABLO_WRODTRANK1_RNG 0x8
#define MCU_WRODTTABLO_WRODTRANK0_RNG 0x0
#define MCU_RDLVLRSP0_RESERVED_RNG 0x12
#define MCU_RDLVLRSP0_STATUS_RNG 0x0
#define MCU_RDLVLRSP1_RESERVED_RNG 0x12
#define MCU_RDLVLRSP1_STATUS_RNG 0x0
#define MCU_WRLVLRSP_RESERVED_RNG 0x12
#define MCU_WRLVLRSP_STATUS_RNG 0x0
#define MCU_RDLVLTMRS_RR_RNG 0x18
#define MCU_RDLVLTMRS_BBGAP_RNG 0x14
#define MCU_RDLVLTMRS_MAX_RNG 0x0
#define MCU_WRLVLTMRS_WW_RNG 0x18
#define MCU_WRLVLTMRS_RESERVED_RNG 0x14
#define MCU_WRLVLTMRS_MAX_RNG 0x0
#define MCU_WRLVLMRD_RESERVED_RNG 0x8
#define MCU_WRLVLMRD_VALUE_RNG 0x0
#define MCU_BISTROWCOL_ROW_RNG 0x10
#define MCU_BISTROWCOL_COL_RNG 0x0
#define MCU_BISTCTLSTS_RD2WRGAP_RNG 0x1b
#define MCU_BISTCTLSTS_WR2RDGAP_RNG 0x16
#define MCU_BISTCTLSTS_WR2WRGAP_RNG 0x12
#define MCU_BISTCTLSTS_RD2RDGAP_RNG 0xe
#define MCU_BISTCTLSTS_WR0RANK_RNG 0xb
#define MCU_BISTCTLSTS_WR1RANK_RNG 0x8
#define MCU_BISTCTLSTS_RD0RANK_RNG 0x5
#define MCU_BISTCTLSTS_RD1RANK_RNG 0x2
#define MCU_BISTCTLSTS_DOWR_RNG 0x1
#define MCU_BISTCTLSTS_DORD_RNG 0x0
#define MCU_DDR_INIT_CTL_CALIBGO_RNG 0x1f
#define MCU_DDR_INIT_CTL_RESERVED_RNG 0xc
#define MCU_DDR_INIT_CTL_RCWAIT_RNG 0x8
#define MCU_DDR_INIT_CTL_RESERVED1_RNG 0x7
#define MCU_DDR_INIT_CTL_REFRESHCNT_RNG 0x4
#define MCU_DDR_INIT_CTL_RESERVED2_RNG 0x1
#define MCU_DDR_INIT_CTL_GO2RESET_RNG 0x0
#define MCU_DDR_INIT_SEQ_RESERVED_RNG 0xe
#define MCU_DDR_INIT_SEQ_RSVD2_RNG 0xd
#define MCU_DDR_INIT_SEQ_PREREFRESHENAB_RNG 0xc
#define MCU_DDR_INIT_SEQ_CLK200ENAB_RNG 0xb
#define MCU_DDR_INIT_SEQ_CLK500ENAB_RNG 0xa
#define MCU_DDR_INIT_SEQ_TXPRENAB_RNG 0x9
#define MCU_DDR_INIT_SEQ_RCENAB_RNG 0x8
#define MCU_DDR_INIT_SEQ_MRENAB_RNG 0x7
#define MCU_DDR_INIT_SEQ_ZQCLENAB_RNG 0x6
#define MCU_DDR_INIT_SEQ_WRLVLENAB_RNG 0x5
#define MCU_DDR_INIT_SEQ_GATEENAB_RNG 0x4
#define MCU_DDR_INIT_SEQ_RDLVLENAB_RNG 0x3
#define MCU_DDR_INIT_SEQ_BISTENAB_RNG 0x2
#define MCU_DDR_INIT_SEQ_CTLUPDATE_RNG 0x1
#define MCU_DDR_INIT_SEQ_SETMEMINITDONE_RNG 0x0
#define MCU_DDR_INIT_STATUS_BISTERROR_RNG 0x1f
#define MCU_DDR_INIT_STATUS_INITCTLSTATE_RNG 0x18
#define MCU_DDR_INIT_STATUS_RESERVED_RNG 0x17
#define MCU_DDR_INIT_STATUS_RETURNSTATE_RNG 0x10
#define MCU_DDR_INIT_STATUS_BADSTATE_RNG 0xf
#define MCU_DDR_INIT_STATUS_TIMEOUTERR_RNG 0xe
#define MCU_DDR_INIT_STATUS_RSVD2_RNG 0xd
#define MCU_DDR_INIT_STATUS_PREREFRESHDONE_RNG 0xc
#define MCU_DDR_INIT_STATUS_CLK200DONE_RNG 0xb
#define MCU_DDR_INIT_STATUS_CLK500DONE_RNG 0xa
#define MCU_DDR_INIT_STATUS_TXPRDONE_RNG 0x9
#define MCU_DDR_INIT_STATUS_RCDONE_RNG 0x8
#define MCU_DDR_INIT_STATUS_MRDONE_RNG 0x7
#define MCU_DDR_INIT_STATUS_ZQCLDONE_RNG 0x6
#define MCU_DDR_INIT_STATUS_WRLVLDONE_RNG 0x5
#define MCU_DDR_INIT_STATUS_GATEDONE_RNG 0x4
#define MCU_DDR_INIT_STATUS_RDLVLDONE_RNG 0x3
#define MCU_DDR_INIT_STATUS_BISTDONE_RNG 0x2
#define MCU_DDR_INIT_STATUS_CTLUPDATEDONE_RNG 0x1
#define MCU_DDR_INIT_STATUS_MEMINITDONE_RNG 0x0
#define MCU_PHY_TRNMODE_RESERVED_RNG 0x13
#define MCU_PHY_TRNMODE_WRLVL_REQ_RNG 0x12
#define MCU_PHY_TRNMODE_RDGATE_REQ_RNG 0x11
#define MCU_PHY_TRNMODE_RDLVL_REQ_RNG 0x10
#define MCU_PHY_TRNMODE_RESERVED1_RNG 0x6
#define MCU_PHY_TRNMODE_WRLVL_MODE_RNG 0x4
#define MCU_PHY_TRNMODE_RDLVL_GATE_MODE_RNG 0x2
#define MCU_PHY_TRNMODE_RDLVL_MODE_RNG 0x0
#define MCU_SWISS_ODTADDR_ENABLE_RNG 0x1f
#define MCU_SWISS_ODTADDR_RESERVED_RNG 0x16
#define MCU_SWISS_ODTADDR_ODTRD_RNG 0x15
#define MCU_SWISS_ODTADDR_ODTWR_RNG 0x14
#define MCU_SWISS_ODTADDR_RESERVED1_RNG 0x13
#define MCU_SWISS_ODTADDR_BANK_RNG 0x10
#define MCU_SWISS_ODTADDR_ADDRESS_RNG 0x0
#define MCU_SWISS_OPCTL_SENDOP_RNG 0x1f
#define MCU_SWISS_OPCTL_RESERVED_RNG 0xd
#define MCU_SWISS_OPCTL_DFI_INIT_RDDATA_EN_RNG 0xc
#define MCU_SWISS_OPCTL_RESERVED1_RNG 0xb
#define MCU_SWISS_OPCTL_DFI_INIT_CMD_N_RNG 0x8
#define MCU_SWISS_OPCTL_DFI_INIT_CS_N_RNG 0x0
#define MCU_SWISS_TRNCTL_ENABLE_RNG 0x1f
#define MCU_SWISS_TRNCTL_RESERVED_RNG 0x13
#define MCU_SWISS_TRNCTL_DFI_WRLVL_EN_RNG 0x12
#define MCU_SWISS_TRNCTL_DFI_RDLVL_GATE_EN_RNG 0x11
#define MCU_SWISS_TRNCTL_DFI_RDLVL_EN_RNG 0x10
#define MCU_SWISS_TRNCTL_DFI_WRLVL_CS_N_RNG 0x8
#define MCU_SWISS_TRNCTL_DFI_RDLVL_CS_N_RNG 0x0
#define MCU_SWISS_TRNWRLOAD_SENDPULSE_RNG 0x1f
#define MCU_SWISS_TRNWRLOAD_RESERVED_RNG 0x12
#define MCU_SWISS_TRNWRLOAD_DFI_WRLVL_LOAD_RNG 0x0
#define MCU_SWISS_TRNWRSTRB_SENDPULSE_RNG 0x1f
#define MCU_SWISS_TRNWRSTRB_RESERVED_RNG 0x12
#define MCU_SWISS_TRNWRSTRB_DFI_WRLVL_STROBE_RNG 0x0
#define MCU_SWISS_TRNRDLOAD_SENDPULSE_RNG 0x1f
#define MCU_SWISS_TRNRDLOAD_RESERVED_RNG 0x12
#define MCU_SWISS_TRNRDLOAD_DFI_RDLVL_LOAD_RNG 0x0
#define MCU_SWISS_RSTCKE_ENABLE_RNG 0x1f
#define MCU_SWISS_RSTCKE_RESERVED_RNG 0x10
#define MCU_SWISS_RSTCKE_DFI_INIT_CKE_RNG 0x8
#define MCU_SWISS_RSTCKE_RESERVED1_RNG 0x1
#define MCU_SWISS_RSTCKE_DFI_RESET_N_RNG 0x0
#define MCU_MCUSLEEPSTS_RANK_0_SLEEPST_RNG 0x1c
#define MCU_MCUSLEEPSTS_RANK_1_SLEEPST_RNG 0x18
#define MCU_MCUSLEEPSTS_RANK_2_SLEEPST_RNG 0x14
#define MCU_MCUSLEEPSTS_RANK_3_SLEEPST_RNG 0x10
#define MCU_MCUSLEEPSTS_RANK_4_SLEEPST_RNG 0xc
#define MCU_MCUSLEEPSTS_RANK_5_SLEEPST_RNG 0x8
#define MCU_MCUSLEEPSTS_RANK_6_SLEEPST_RNG 0x4
#define MCU_MCUSLEEPSTS_RANK_7_SLEEPST_RNG 0x0
#define MCU_MCUSLEEPCTL_WAKEUP_RNG 0x1f
#define MCU_MCUSLEEPCTL_RESERVED_RNG 0x1a
#define MCU_MCUSLEEPCTL_PAIRRANKS0145_RNG 0x19
#define MCU_MCUSLEEPCTL_PAIRRANKS2367_RNG 0x18
#define MCU_MCUSLEEPCTL_RESERVED1_RNG 0x0
#define MCU_MCULSCTL_LSENABLED_RNG 0x1f
#define MCU_MCULSCTL_FORCELS_RNG 0x1e
#define MCU_MCULSCTL_RESERVED_RNG 0x17
#define MCU_MCULSCTL_LSTHRESHSEL_RNG 0x14
#define MCU_MCULSCTL_RESERVED1_RNG 0xf
#define MCU_MCULSCTL_LSREENTRYSEL_RNG 0xc
#define MCU_MCULSCTL_RESERVED2_RNG 0x0
#define MCU_MCUDSCTL_DSENABLED_RNG 0x1f
#define MCU_MCUDSCTL_FORCEDS_RNG 0x1e
#define MCU_MCUDSCTL_RESUMEDS_RNG 0x1d
#define MCU_MCUDSCTL_RESERVED_RNG 0x1a
#define MCU_MCUDSCTL_ZQCONSRX_RNG 0x18
#define MCU_MCUDSCTL_DSTHRESHSEL_RNG 0x14
#define MCU_MCUDSCTL_RESERVED1_RNG 0x10
#define MCU_MCUDSCTL_DSREENTRYSEL_RNG 0xc
#define MCU_MCUDSCTL_RESERVED2_RNG 0x0
#define MCU_MCUZQCCTL_ENABLE_RNG 0x1f
#define MCU_MCUZQCCTL_INTERVAL_RNG 0x0
#define MCU_MCUZQCDLY_RESERVED_RNG 0x5
#define MCU_MCUZQCDLY_VALUE_RNG 0x0
#define MCU_RANK_SRXNODLL_RESERVED_RNG 0x9
#define MCU_RANK_SRXNODLL_TIMER_VALUE_RNG 0x0
#define MCU_RANK_SRXDLL_RESERVED_RNG 0xa
#define MCU_RANK_SRXDLL_TIMER_VALUE_RNG 0x0
#define MCU_RANK_PDXNODLL_RESERVED_RNG 0x3
#define MCU_RANK_PDXNODLL_TIMER_VALUE_RNG 0x0
#define MCU_RANK_PDXDLL_RESERVED_RNG 0x5
#define MCU_RANK_PDXDLL_TIMER_VALUE_RNG 0x0
#define MCU_RANK_RDPDE_RESERVED_RNG 0x6
#define MCU_RANK_RDPDE_TIMER_VALUE_RNG 0x0
#define MCU_RANK_WRPDE_RESERVED_RNG 0x6
#define MCU_RANK_WRPDE_TIMER_VALUE_RNG 0x0
#define MCU_RANK_WRAPDE_RESERVED_RNG 0x6
#define MCU_RANK_WRAPDE_TIMER_VALUE_RNG 0x0
#define MCU_RANK_CKEMIN_RESERVED_RNG 0x3
#define MCU_RANK_CKEMIN_TIMER_VALUE_RNG 0x0
#define MCU_RANK_ODTL_RESERVED_RNG 0x5
#define MCU_RANK_ODTL_TIMER_VALUE_RNG 0x0
#define MCU_RAMSLEEPCTL_DSENABLE_RNG 0x1f
#define MCU_RAMSLEEPCTL_LSENABLE_RNG 0x1e
#define MCU_RAMSLEEPCTL_RESERVED_RNG 0x18
#define MCU_RAMSLEEPCTL_DSTHRESHOLD_RNG 0x14
#define MCU_RAMSLEEPCTL_LSTHRESHOLD_RNG 0x10
#define MCU_RAMSLEEPCTL_DSWAKECNT_RNG 0x8
#define MCU_RAMSLEEPCTL_LSWAKECNT_RNG 0x0
#define MCU_RAMSHUTDOWN_RESERVED_RNG 0x1
#define MCU_RAMSHUTDOWN_SHUTDOWN_RNG 0x0
#define MCU_RAMRWMCTL_RMA_RNG 0x1c
#define MCU_RAMRWMCTL_RESERVED_RNG 0x19
#define MCU_RAMRWMCTL_RMEA_RNG 0x18
#define MCU_RAMRWMCTL_RMB_RNG 0x14
#define MCU_RAMRWMCTL_RESERVED1_RNG 0x11
#define MCU_RAMRWMCTL_RMEB_RNG 0x10
#define MCU_RAMRWMCTL_RESERVED2_RNG 0x0
#define MCU_PHYLPCTL_ENABLE_RNG 0x1f
#define MCU_PHYLPCTL_RESERVED_RNG 0x18
#define MCU_PHYLPCTL_DSDELAYCNT_RNG 0x14
#define MCU_PHYLPCTL_LSDELAYCNT_RNG 0x10
#define MCU_PHYLPCTL_RESERVED1_RNG 0xe
#define MCU_PHYLPCTL_MINAWAKECNT_RNG 0x8
#define MCU_PHYLPCTL_DSWAKECNT_RNG 0x4
#define MCU_PHYLPCTL_LSWAKECNT_RNG 0x0
#define MCU_RAMPHYPWRSTS_RESERVED_RNG 0xa
#define MCU_RAMPHYPWRSTS_PHYLPREQ_RNG 0x9
#define MCU_RAMPHYPWRSTS_PHYLPACK_RNG 0x8
#define MCU_RAMPHYPWRSTS_RESERVED1_RNG 0x7
#define MCU_RAMPHYPWRSTS_PHYSLEEPSTATE_RNG 0x4
#define MCU_RAMPHYPWRSTS_RESERVED2_RNG 0x2
#define MCU_RAMPHYPWRSTS_RAMSLEEPSTATE_RNG 0x0
#define MCU_RANK_0_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_0_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_0_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_0_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_0_BASE_RESERVED_RNG 0x13
#define MCU_RANK_0_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_0_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_0_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_0_MASK_RESERVED_RNG 0x13
#define MCU_RANK_0_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_0_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_0_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_0_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_0_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR0_RESERVED_RNG 0x4
#define MCU_MCUESRR0_MULTUCERR_RNG 0x3
#define MCU_MCUESRR0_BACKUCERR_RNG 0x2
#define MCU_MCUESRR0_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR0_CERR_RNG 0x0
#define MCU_MCUESRRA0_RESERVED_RNG 0x4
#define MCU_MCUESRRA0_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA0_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA0_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA0_CERR_RNG 0x0
#define MCU_MCUEBLRR0_RESERVED_RNG 0x3
#define MCU_MCUEBLRR0_ERRBANK_RNG 0x0
#define MCU_MCUERCRR0_ERRROW_RNG 0x10
#define MCU_MCUERCRR0_RESERVED_RNG 0xc
#define MCU_MCUERCRR0_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT0_RESERVED_RNG 0x10
#define MCU_MCUSBECNT0_COUNT_RNG 0x0
#define MCU_RANK_1_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_1_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_1_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_1_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_1_BASE_RESERVED_RNG 0x13
#define MCU_RANK_1_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_1_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_1_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_1_MASK_RESERVED_RNG 0x13
#define MCU_RANK_1_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_1_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_1_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_1_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_1_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR1_RESERVED_RNG 0x4
#define MCU_MCUESRR1_MULTUCERR_RNG 0x3
#define MCU_MCUESRR1_BACKUCERR_RNG 0x2
#define MCU_MCUESRR1_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR1_CERR_RNG 0x0
#define MCU_MCUESRRA1_RESERVED_RNG 0x4
#define MCU_MCUESRRA1_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA1_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA1_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA1_CERR_RNG 0x0
#define MCU_MCUEBLRR1_RESERVED_RNG 0x3
#define MCU_MCUEBLRR1_ERRBANK_RNG 0x0
#define MCU_MCUERCRR1_ERRROW_RNG 0x10
#define MCU_MCUERCRR1_RESERVED_RNG 0xc
#define MCU_MCUERCRR1_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT1_RESERVED_RNG 0x10
#define MCU_MCUSBECNT1_COUNT_RNG 0x0
#define MCU_RANK_2_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_2_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_2_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_2_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_2_BASE_RESERVED_RNG 0x13
#define MCU_RANK_2_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_2_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_2_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_2_MASK_RESERVED_RNG 0x13
#define MCU_RANK_2_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_2_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_2_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_2_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_2_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR2_RESERVED_RNG 0x4
#define MCU_MCUESRR2_MULTUCERR_RNG 0x3
#define MCU_MCUESRR2_BACKUCERR_RNG 0x2
#define MCU_MCUESRR2_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR2_CERR_RNG 0x0
#define MCU_MCUESRRA2_RESERVED_RNG 0x4
#define MCU_MCUESRRA2_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA2_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA2_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA2_CERR_RNG 0x0
#define MCU_MCUEBLRR2_RESERVED_RNG 0x3
#define MCU_MCUEBLRR2_ERRBANK_RNG 0x0
#define MCU_MCUERCRR2_ERRROW_RNG 0x10
#define MCU_MCUERCRR2_RESERVED_RNG 0xc
#define MCU_MCUERCRR2_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT2_RESERVED_RNG 0x10
#define MCU_MCUSBECNT2_COUNT_RNG 0x0
#define MCU_RANK_3_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_3_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_3_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_3_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_3_BASE_RESERVED_RNG 0x13
#define MCU_RANK_3_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_3_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_3_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_3_MASK_RESERVED_RNG 0x13
#define MCU_RANK_3_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_3_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_3_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_3_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_3_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR3_RESERVED_RNG 0x4
#define MCU_MCUESRR3_MULTUCERR_RNG 0x3
#define MCU_MCUESRR3_BACKUCERR_RNG 0x2
#define MCU_MCUESRR3_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR3_CERR_RNG 0x0
#define MCU_MCUESRRA3_RESERVED_RNG 0x4
#define MCU_MCUESRRA3_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA3_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA3_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA3_CERR_RNG 0x0
#define MCU_MCUEBLRR3_RESERVED_RNG 0x3
#define MCU_MCUEBLRR3_ERRBANK_RNG 0x0
#define MCU_MCUERCRR3_ERRROW_RNG 0x10
#define MCU_MCUERCRR3_RESERVED_RNG 0xc
#define MCU_MCUERCRR3_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT3_RESERVED_RNG 0x10
#define MCU_MCUSBECNT3_COUNT_RNG 0x0
#define MCU_RANK_4_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_4_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_4_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_4_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_4_BASE_RESERVED_RNG 0x13
#define MCU_RANK_4_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_4_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_4_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_4_MASK_RESERVED_RNG 0x13
#define MCU_RANK_4_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_4_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_4_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_4_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_4_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR4_RESERVED_RNG 0x4
#define MCU_MCUESRR4_MULTUCERR_RNG 0x3
#define MCU_MCUESRR4_BACKUCERR_RNG 0x2
#define MCU_MCUESRR4_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR4_CERR_RNG 0x0
#define MCU_MCUESRRA4_RESERVED_RNG 0x4
#define MCU_MCUESRRA4_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA4_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA4_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA4_CERR_RNG 0x0
#define MCU_MCUEBLRR4_RESERVED_RNG 0x3
#define MCU_MCUEBLRR4_ERRBANK_RNG 0x0
#define MCU_MCUERCRR4_ERRROW_RNG 0x10
#define MCU_MCUERCRR4_RESERVED_RNG 0xc
#define MCU_MCUERCRR4_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT4_RESERVED_RNG 0x10
#define MCU_MCUSBECNT4_COUNT_RNG 0x0
#define MCU_RANK_5_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_5_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_5_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_5_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_5_BASE_RESERVED_RNG 0x13
#define MCU_RANK_5_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_5_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_5_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_5_MASK_RESERVED_RNG 0x13
#define MCU_RANK_5_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_5_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_5_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_5_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_5_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR5_RESERVED_RNG 0x4
#define MCU_MCUESRR5_MULTUCERR_RNG 0x3
#define MCU_MCUESRR5_BACKUCERR_RNG 0x2
#define MCU_MCUESRR5_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR5_CERR_RNG 0x0
#define MCU_MCUESRRA5_RESERVED_RNG 0x4
#define MCU_MCUESRRA5_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA5_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA5_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA5_CERR_RNG 0x0
#define MCU_MCUEBLRR5_RESERVED_RNG 0x3
#define MCU_MCUEBLRR5_ERRBANK_RNG 0x0
#define MCU_MCUERCRR5_ERRROW_RNG 0x10
#define MCU_MCUERCRR5_RESERVED_RNG 0xc
#define MCU_MCUERCRR5_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT5_RESERVED_RNG 0x10
#define MCU_MCUSBECNT5_COUNT_RNG 0x0
#define MCU_RANK_6_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_6_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_6_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_6_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_6_BASE_RESERVED_RNG 0x13
#define MCU_RANK_6_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_6_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_6_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_6_MASK_RESERVED_RNG 0x13
#define MCU_RANK_6_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_6_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_6_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_6_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_6_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR6_RESERVED_RNG 0x4
#define MCU_MCUESRR6_MULTUCERR_RNG 0x3
#define MCU_MCUESRR6_BACKUCERR_RNG 0x2
#define MCU_MCUESRR6_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR6_CERR_RNG 0x0
#define MCU_MCUESRRA6_RESERVED_RNG 0x4
#define MCU_MCUESRRA6_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA6_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA6_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA6_CERR_RNG 0x0
#define MCU_MCUEBLRR6_RESERVED_RNG 0x3
#define MCU_MCUEBLRR6_ERRBANK_RNG 0x0
#define MCU_MCUERCRR6_ERRROW_RNG 0x10
#define MCU_MCUERCRR6_RESERVED_RNG 0xc
#define MCU_MCUERCRR6_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT6_RESERVED_RNG 0x10
#define MCU_MCUSBECNT6_COUNT_RNG 0x0
#define MCU_RANK_7_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_7_SIZE_RANK_SIZE_RNG 0x0
#define MCU_RANK_7_CONFIG_RESERVED_RNG 0x1
#define MCU_RANK_7_CONFIG_RANK_CONFIGURATION_RNG 0x0
#define MCU_RANK_7_BASE_RESERVED_RNG 0x13
#define MCU_RANK_7_BASE_RANK_BASE_LOWER_RNG 0x10
#define MCU_RANK_7_BASE_RESERVED1_RNG 0xe
#define MCU_RANK_7_BASE_RANK_BASE_UPPER_RNG 0x0
#define MCU_RANK_7_MASK_RESERVED_RNG 0x13
#define MCU_RANK_7_MASK_RANK_MASK_LOWER_RNG 0x10
#define MCU_RANK_7_MASK_RESERVED1_RNG 0xe
#define MCU_RANK_7_MASK_RANK_MASK_UPPER_RNG 0x0
#define MCU_RANK_7_HASH_SIZE_RESERVED_RNG 0x3
#define MCU_RANK_7_HASH_SIZE_HASH_SIZE_RNG 0x0
#define MCU_MCUESRR7_RESERVED_RNG 0x4
#define MCU_MCUESRR7_MULTUCERR_RNG 0x3
#define MCU_MCUESRR7_BACKUCERR_RNG 0x2
#define MCU_MCUESRR7_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRR7_CERR_RNG 0x0
#define MCU_MCUESRRA7_RESERVED_RNG 0x4
#define MCU_MCUESRRA7_MULTUCERR_RNG 0x3
#define MCU_MCUESRRA7_BACKUCERR_RNG 0x2
#define MCU_MCUESRRA7_DEMANDUCERR_RNG 0x1
#define MCU_MCUESRRA7_CERR_RNG 0x0
#define MCU_MCUEBLRR7_RESERVED_RNG 0x3
#define MCU_MCUEBLRR7_ERRBANK_RNG 0x0
#define MCU_MCUERCRR7_ERRROW_RNG 0x10
#define MCU_MCUERCRR7_RESERVED_RNG 0xc
#define MCU_MCUERCRR7_ERRBANK_RNG 0x0
#define MCU_MCUSBECNT7_RESERVED_RNG 0x10
#define MCU_MCUSBECNT7_COUNT_RNG 0x0
#define MCU_RDLVLRSPSTG1LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG1HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG2LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG2HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG3LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG3HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG4LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG4HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG5LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG5HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG6LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG6HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG7LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG7HI_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG8LO_VALUE_RNG 0x0
#define MCU_RDLVLRSPSTG8HI_VALUE_RNG 0x0
#define MCU_BISTWRDATA0_VALUE_RNG 0x0
#define MCU_BISTWRDATA1_VALUE_RNG 0x0
#define MCU_BISTWRDATA2_VALUE_RNG 0x0
#define MCU_BISTWRDATA3_VALUE_RNG 0x0
#define MCU_BISTWRDATA4_RESERVED_RNG 0x10
#define MCU_BISTWRDATA4_VALUE1_RNG 0x8
#define MCU_BISTWRDATA4_VALUE0_RNG 0x0

#endif
